/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/*
 * AddQuestionDialog.java
 *
 * Created on May 7, 2010, 3:41:29 PM
 */

package lecturequiz.teacherclient;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Enumeration;
import java.util.List;
import java.util.Stack;
import java.util.Vector;
import javax.swing.AbstractButton;
import javax.swing.ButtonGroup;
import javax.swing.JOptionPane;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.Timer;
import javax.xml.ws.Holder;
import lecturequiz.server.service.Answer;
import lecturequiz.server.service.FullQuestionInfo;
import lecturequiz.teacherclient.service.ConnectionManager;
import lecturequiz.teacherclient.utils.ComboItem;

/**
 *
 * @author avenger
 */
public class AddQuestionDialog extends javax.swing.JDialog {
    private int dialogResult;

    private Timer filterTimer;

    private boolean hasLoadedQuestionList;

    private ButtonGroup correctAnswerGroup;

    private JRadioButton[] correctAnswerButtons;
    private JTextField[] answerTexts;

    private FullQuestionInfo question = null;

    private Vector<ComboItem<Integer>> questionItems;

    /** Creates new form AddQuestionDialog */
    public AddQuestionDialog(java.awt.Frame parent, boolean modal) {
        super(parent, modal);
        initComponents();

        dialogResult = JOptionPane.CANCEL_OPTION;

        correctAnswerGroup = new ButtonGroup();
        correctAnswerGroup.add(radCorrect1);
        correctAnswerGroup.add(radCorrect2);
        correctAnswerGroup.add(radCorrect3);
        correctAnswerGroup.add(radCorrect4);

        answerTexts = new JTextField[4];
        answerTexts[0] = txtAnswer1;
        answerTexts[1] = txtAnswer2;
        answerTexts[2] = txtAnswer3;
        answerTexts[3] = txtAnswer4;

        hasLoadedQuestionList = false;

        filterTimer = new Timer(1000, new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                filterQuestions();
            }
        });
    }

    /**
     * Returns the dialog result of this dialog
     * @return
     */
    public int getDialogResult() {
        return dialogResult;
    }

    /**
     * Override the setVisible method to reset the dialogResult on show,
     * @param b True to show the dialog, false to hide it.
     */
    @Override
    public void setVisible(boolean b) {

        if(b) // Reset the dialog result
            dialogResult = JOptionPane.CANCEL_OPTION;
        
        super.setVisible(b);
    }

    /**
     * Returns the question selected by this dialog
     * @return A FullQuesztionInfo-object with the question selected. If a new
     * question was created, the id of this question will be 0
     */
    public FullQuestionInfo getQuestion() {
        return question;
    }

    private boolean validateInput() {
        if(jTabbedPane1.getSelectedIndex() == 0) { // New question

            if(correctAnswerGroup.getSelection() == null) {
                JOptionPane.showMessageDialog(this,"A correct answer must be selected.","Lecture Quiz 2.0",JOptionPane.ERROR_MESSAGE);
                return false;
            }

            question = new FullQuestionInfo();

            question.setId(0);
            question.setText(txtQuestionText.getText());

            int timeout = (Integer)questionTimeout.getValue();
            if(timeout < 1) {
                JOptionPane.showMessageDialog(this, "The timeout value is invalid.","Lecture Quiz 2.0",JOptionPane.ERROR_MESSAGE);
                return false;
            }
            question.setTimeout(timeout);
            question.setTimeleft(timeout);

            int i = 0;
            for(Enumeration<AbstractButton> e = correctAnswerGroup.getElements(); e.hasMoreElements(); i++) {
                if(e.nextElement().isSelected()) {
                    question.setCorrectAnswer(i);
                    break;
                }
            }

            // Add answers
            for(i = 0; i < answerTexts.length; i++) {
                Answer a = new Answer();
                a.setId(i);
                a.setText(answerTexts[i].getText());
                question.getAnswers().add(a);
            }
        } else {
            ComboItem<Integer> item = (ComboItem<Integer>)questionList.getSelectedValue();
            if(item == null) {
                JOptionPane.showMessageDialog(this, "No question selected","Lecture Quiz 2.0",JOptionPane.ERROR_MESSAGE);
                return false;
            }



            Holder<FullQuestionInfo> nfo = new Holder<FullQuestionInfo>();
            int ret;
            if((ret = ConnectionManager.getInstance().getPort().getQuestion(item.getId(), nfo)) != 0) {
                JOptionPane.showMessageDialog(this, "Could not get question from server. Error code: " + ret,"Lecture Quiz 2.0",JOptionPane.ERROR_MESSAGE);
                question = null;
                return false;
            }

            question = nfo.value;
        }

        return true;
    }

    private void filterQuestions() {
        if(txtQuestionFilter.getText().trim().isEmpty()) {
            questionList.setListData(questionItems);
            return;
        }

        Vector<ComboItem<Integer>> filteredItems = new Vector<ComboItem<Integer>>();
        for(int i = 0; i < questionItems.size(); i++) {
            if(questionItems.get(i).getText().contains(txtQuestionFilter.getText().trim())) {
                filteredItems.add(questionItems.get(i));
            }
        }

        questionList.setListData(filteredItems);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        btnCancel = new javax.swing.JButton();
        btnOK = new javax.swing.JButton();
        jTabbedPane1 = new javax.swing.JTabbedPane();
        newQuestionPane = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        txtQuestionText = new javax.swing.JTextField();
        jLabel2 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();
        jLabel6 = new javax.swing.JLabel();
        txtAnswer1 = new javax.swing.JTextField();
        txtAnswer2 = new javax.swing.JTextField();
        txtAnswer3 = new javax.swing.JTextField();
        radCorrect1 = new javax.swing.JRadioButton();
        radCorrect2 = new javax.swing.JRadioButton();
        radCorrect3 = new javax.swing.JRadioButton();
        radCorrect4 = new javax.swing.JRadioButton();
        txtAnswer4 = new javax.swing.JTextField();
        jLabel7 = new javax.swing.JLabel();
        jLabel8 = new javax.swing.JLabel();
        questionTimeout = new javax.swing.JSpinner();
        jLabel9 = new javax.swing.JLabel();
        existingQuestionPane = new javax.swing.JPanel();
        jLabel10 = new javax.swing.JLabel();
        txtQuestionFilter = new javax.swing.JTextField();
        jScrollPane1 = new javax.swing.JScrollPane();
        questionList = new javax.swing.JList();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setName("Form"); // NOI18N

        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(lecturequiz.teacherclient.LQTeacherClientApp.class).getContext().getResourceMap(AddQuestionDialog.class);
        btnCancel.setText(resourceMap.getString("btnCancel.text")); // NOI18N
        btnCancel.setName("btnCancel"); // NOI18N
        btnCancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnCancelActionPerformed(evt);
            }
        });

        btnOK.setText(resourceMap.getString("btnOK.text")); // NOI18N
        btnOK.setName("btnOK"); // NOI18N
        btnOK.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnOKActionPerformed(evt);
            }
        });

        jTabbedPane1.setName("jTabbedPane1"); // NOI18N
        jTabbedPane1.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                jTabbedPane1StateChanged(evt);
            }
        });

        newQuestionPane.setName("newQuestionPane"); // NOI18N

        jLabel1.setText(resourceMap.getString("jLabel1.text")); // NOI18N
        jLabel1.setName("jLabel1"); // NOI18N

        txtQuestionText.setText(resourceMap.getString("txtQuestionText.text")); // NOI18N
        txtQuestionText.setName("txtQuestionText"); // NOI18N

        jLabel2.setText(resourceMap.getString("jLabel2.text")); // NOI18N
        jLabel2.setName("jLabel2"); // NOI18N

        jLabel3.setText(resourceMap.getString("jLabel3.text")); // NOI18N
        jLabel3.setName("jLabel3"); // NOI18N

        jLabel4.setText(resourceMap.getString("jLabel4.text")); // NOI18N
        jLabel4.setName("jLabel4"); // NOI18N

        jLabel5.setText(resourceMap.getString("jLabel5.text")); // NOI18N
        jLabel5.setName("jLabel5"); // NOI18N

        jLabel6.setText(resourceMap.getString("jLabel6.text")); // NOI18N
        jLabel6.setName("jLabel6"); // NOI18N

        txtAnswer1.setText(resourceMap.getString("txtAnswer1.text")); // NOI18N
        txtAnswer1.setName("txtAnswer1"); // NOI18N

        txtAnswer2.setText(resourceMap.getString("txtAnswer2.text")); // NOI18N
        txtAnswer2.setName("txtAnswer2"); // NOI18N

        txtAnswer3.setText(resourceMap.getString("txtAnswer3.text")); // NOI18N
        txtAnswer3.setName("txtAnswer3"); // NOI18N

        radCorrect1.setName("radCorrect1"); // NOI18N

        radCorrect2.setName("radCorrect2"); // NOI18N

        radCorrect3.setName("radCorrect3"); // NOI18N

        radCorrect4.setName("radCorrect4"); // NOI18N

        txtAnswer4.setText(resourceMap.getString("txtAnswer4.text")); // NOI18N
        txtAnswer4.setName("txtAnswer4"); // NOI18N

        jLabel7.setText(resourceMap.getString("jLabel7.text")); // NOI18N
        jLabel7.setName("jLabel7"); // NOI18N

        jLabel8.setText(resourceMap.getString("jLabel8.text")); // NOI18N
        jLabel8.setName("jLabel8"); // NOI18N

        questionTimeout.setName("questionTimeout"); // NOI18N
        questionTimeout.setValue(30);

        jLabel9.setText(resourceMap.getString("jLabel9.text")); // NOI18N
        jLabel9.setName("jLabel9"); // NOI18N

        javax.swing.GroupLayout newQuestionPaneLayout = new javax.swing.GroupLayout(newQuestionPane);
        newQuestionPane.setLayout(newQuestionPaneLayout);
        newQuestionPaneLayout.setHorizontalGroup(
            newQuestionPaneLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(newQuestionPaneLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(newQuestionPaneLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(newQuestionPaneLayout.createSequentialGroup()
                        .addComponent(jLabel1)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 365, Short.MAX_VALUE)
                        .addComponent(jLabel8)
                        .addContainerGap())
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, newQuestionPaneLayout.createSequentialGroup()
                        .addComponent(txtQuestionText, javax.swing.GroupLayout.DEFAULT_SIZE, 479, Short.MAX_VALUE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(questionTimeout, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jLabel9)
                        .addContainerGap())
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, newQuestionPaneLayout.createSequentialGroup()
                        .addGroup(newQuestionPaneLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addGroup(javax.swing.GroupLayout.Alignment.LEADING, newQuestionPaneLayout.createSequentialGroup()
                                .addComponent(jLabel2)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 471, Short.MAX_VALUE)
                                .addComponent(jLabel7))
                            .addGroup(newQuestionPaneLayout.createSequentialGroup()
                                .addGroup(newQuestionPaneLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(jLabel5)
                                    .addComponent(jLabel6)
                                    .addGroup(newQuestionPaneLayout.createSequentialGroup()
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                        .addComponent(jLabel3))
                                    .addComponent(jLabel4))
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addGroup(newQuestionPaneLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(txtAnswer3, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 534, Short.MAX_VALUE)
                                    .addComponent(txtAnswer2, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 534, Short.MAX_VALUE)
                                    .addComponent(txtAnswer1, javax.swing.GroupLayout.DEFAULT_SIZE, 534, Short.MAX_VALUE)
                                    .addComponent(txtAnswer4, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 534, Short.MAX_VALUE))
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addGroup(newQuestionPaneLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                                    .addGroup(newQuestionPaneLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                        .addComponent(radCorrect2)
                                        .addGroup(newQuestionPaneLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                                            .addComponent(radCorrect1)
                                            .addComponent(radCorrect3)))
                                    .addComponent(radCorrect4))))
                        .addContainerGap())))
        );
        newQuestionPaneLayout.setVerticalGroup(
            newQuestionPaneLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(newQuestionPaneLayout.createSequentialGroup()
                .addGroup(newQuestionPaneLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel1)
                    .addComponent(jLabel8))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(newQuestionPaneLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(txtQuestionText, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(questionTimeout, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel9))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(newQuestionPaneLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel2)
                    .addComponent(jLabel7))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(newQuestionPaneLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(newQuestionPaneLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(txtAnswer1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(jLabel3))
                    .addComponent(radCorrect1))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(newQuestionPaneLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(newQuestionPaneLayout.createSequentialGroup()
                        .addGroup(newQuestionPaneLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(txtAnswer2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(jLabel4))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(newQuestionPaneLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabel5)
                            .addComponent(txtAnswer3, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                    .addGroup(newQuestionPaneLayout.createSequentialGroup()
                        .addComponent(radCorrect2)
                        .addGap(1, 1, 1)
                        .addComponent(radCorrect3)))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(newQuestionPaneLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(radCorrect4)
                    .addGroup(newQuestionPaneLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(jLabel6)
                        .addComponent(txtAnswer4, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap(130, Short.MAX_VALUE))
        );

        jTabbedPane1.addTab(resourceMap.getString("newQuestionPane.TabConstraints.tabTitle"), newQuestionPane); // NOI18N

        existingQuestionPane.setName("existingQuestionPane"); // NOI18N

        jLabel10.setText(resourceMap.getString("jLabel10.text")); // NOI18N
        jLabel10.setName("jLabel10"); // NOI18N

        txtQuestionFilter.setText(resourceMap.getString("txtQuestionFilter.text")); // NOI18N
        txtQuestionFilter.setName("txtQuestionFilter"); // NOI18N
        txtQuestionFilter.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyTyped(java.awt.event.KeyEvent evt) {
                txtQuestionFilterKeyTyped(evt);
            }
        });

        jScrollPane1.setName("jScrollPane1"); // NOI18N

        questionList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        questionList.setName("questionList"); // NOI18N
        jScrollPane1.setViewportView(questionList);

        javax.swing.GroupLayout existingQuestionPaneLayout = new javax.swing.GroupLayout(existingQuestionPane);
        existingQuestionPane.setLayout(existingQuestionPaneLayout);
        existingQuestionPaneLayout.setHorizontalGroup(
            existingQuestionPaneLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(existingQuestionPaneLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(existingQuestionPaneLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 596, Short.MAX_VALUE)
                    .addGroup(existingQuestionPaneLayout.createSequentialGroup()
                        .addGroup(existingQuestionPaneLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabel10)
                            .addComponent(txtQuestionFilter, javax.swing.GroupLayout.PREFERRED_SIZE, 136, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addContainerGap(460, Short.MAX_VALUE))))
        );
        existingQuestionPaneLayout.setVerticalGroup(
            existingQuestionPaneLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(existingQuestionPaneLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel10)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(txtQuestionFilter, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 234, Short.MAX_VALUE))
        );

        jTabbedPane1.addTab(resourceMap.getString("existingQuestionPane.TabConstraints.tabTitle"), existingQuestionPane); // NOI18N

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 613, Short.MAX_VALUE)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addContainerGap(460, Short.MAX_VALUE)
                .addComponent(btnOK)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(btnCancel)
                .addContainerGap())
            .addComponent(jTabbedPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 613, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 358, Short.MAX_VALUE)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                .addComponent(jTabbedPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 316, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(btnCancel)
                    .addComponent(btnOK))
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void btnOKActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnOKActionPerformed
        if(!validateInput())
            return;
        
        dialogResult = JOptionPane.OK_OPTION;

        setVisible(false);
    }//GEN-LAST:event_btnOKActionPerformed

    private void jTabbedPane1StateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_jTabbedPane1StateChanged
        // Load the list of questions if it hasn't already been loaded
        if(!hasLoadedQuestionList && jTabbedPane1.getSelectedIndex() == 1) {
            Holder<List<FullQuestionInfo>> info = new Holder<List<FullQuestionInfo>>();
            int ret;
            if((ret = ConnectionManager.getInstance().getPort().getQuestionList(info)) != 0) {
                JOptionPane.showMessageDialog(this, "Failed to get list of questions from server. Error code: " + ret,
                        "Lecture Quiz 2.0", JOptionPane.ERROR_MESSAGE);
                return;
            }

            questionItems = new Vector<ComboItem<Integer>>();

            for(int i = 0; i < info.value.size(); i++) {
                FullQuestionInfo fqi = info.value.get(i);
                questionItems.add(new ComboItem<Integer>(fqi.getText() + " - " + fqi.getAnswers().get(0).getText(),
                        fqi.getId()));
            }

            // Update the question list and apply the filter
            filterQuestions();

            hasLoadedQuestionList = true;
        }
    }//GEN-LAST:event_jTabbedPane1StateChanged

    private void btnCancelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnCancelActionPerformed
        dialogResult = JOptionPane.CANCEL_OPTION;
        setVisible(false);
    }//GEN-LAST:event_btnCancelActionPerformed

    private void txtQuestionFilterKeyTyped(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_txtQuestionFilterKeyTyped
        filterTimer.restart();
    }//GEN-LAST:event_txtQuestionFilterKeyTyped

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnCancel;
    private javax.swing.JButton btnOK;
    private javax.swing.JPanel existingQuestionPane;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JTabbedPane jTabbedPane1;
    private javax.swing.JPanel newQuestionPane;
    private javax.swing.JList questionList;
    private javax.swing.JSpinner questionTimeout;
    private javax.swing.JRadioButton radCorrect1;
    private javax.swing.JRadioButton radCorrect2;
    private javax.swing.JRadioButton radCorrect3;
    private javax.swing.JRadioButton radCorrect4;
    private javax.swing.JTextField txtAnswer1;
    private javax.swing.JTextField txtAnswer2;
    private javax.swing.JTextField txtAnswer3;
    private javax.swing.JTextField txtAnswer4;
    private javax.swing.JTextField txtQuestionFilter;
    private javax.swing.JTextField txtQuestionText;
    // End of variables declaration//GEN-END:variables

}
