/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/*
 * MainMenu.java
 *
 * Created on Mar 2, 2010, 2:35:33 PM
 */

package lecturequiz.teacherclient;

import java.awt.CardLayout;
import java.util.List;
import javax.swing.JOptionPane;
import javax.xml.ws.Holder;
import lecturequiz.server.service.LectureQuizService;
import lecturequiz.server.service.QuizInfo;
import lecturequiz.server.service.GameModeInfo;
import lecturequiz.teacherclient.gamemodes.AbstractGameMode;
import lecturequiz.teacherclient.gamemodes.GameManager;
import lecturequiz.teacherclient.gamemodes.IGameModeFactory;
import lecturequiz.teacherclient.service.ConnectionManager;
import lecturequiz.teacherclient.utils.ComboItem;

/**
 *
 * @author avenger
 */
public class MainMenu extends javax.swing.JPanel {

    public enum PANEL {
        NO_PANEL,
        LOGIN_PANEL,
        MAIN_PANEL,
        START_QUIZ_PANEL,
        GAME_PANEL,
        EDIT_QUIZ_PANEL
    }

    private PANEL currentPanel;
    private QuizEditor quizEditorPanel = null;

    private static MainMenu _instance = null;

    /** Creates new form MainMenu */
    private MainMenu() {
        initComponents();

        currentPanel = PANEL.NO_PANEL;
    }

    public static synchronized MainMenu getInstance() {
        if(_instance == null)
            _instance = new MainMenu();
        return _instance;
    }


    public void setPanel(PANEL panel) {

        // Perform cleanup if the previous panel needed it
        switch(currentPanel) {
            case GAME_PANEL:
                // Destroy the previous game
                GameManager.getInstance().destroyGame();
                break;

            default: // Default is no clean up
        }

        // Switch to the new panel
        switch(panel) {
            case LOGIN_PANEL:
                ((CardLayout)this.getLayout()).show(this, "loginPanel");
                break;

            case MAIN_PANEL:
                ((CardLayout)this.getLayout()).show(this, "mainPanel");
                break;

            case START_QUIZ_PANEL:
                if(!prepareStartQuizPanel())
                    return;

                ((CardLayout)this.getLayout()).show(this, "startQuizPanel");
                break;

            case GAME_PANEL:
                ((CardLayout)this.getLayout()).show(this, "gamePanel");
                break;

            case EDIT_QUIZ_PANEL:
                // Create the quiz editor panel if it's not already created
                if(quizEditorPanel == null) {
                    quizEditorPanel = new QuizEditor();
                    this.add(quizEditorPanel, "editQuizPanel");
                }
                
                if(!quizEditorPanel.init())
                    return;

                ((CardLayout)this.getLayout()).show(this,"editQuizPanel");
                break;
        }

        currentPanel = panel;
    }



    private boolean prepareStartQuizPanel() {
        // Get the list of quizzes from the server
        quizCombo.removeAllItems();
        Holder<List<QuizInfo>> quizzes = new Holder<List<QuizInfo>> ();
        Holder<List<GameModeInfo>> serverModes = new Holder<List<GameModeInfo>> ();

        if(ConnectionManager.getInstance().getPort().getAvailableQuizzes(quizzes) != 0) {
            JOptionPane.showMessageDialog(this, "Could not get a list of quizzes");
            return false;
        }

        for(int i = 0; i < quizzes.value.size(); i++)
            quizCombo.addItem(new ComboItem<Integer>(quizzes.value.get(i).getName(),quizzes.value.get(i).getId()));

        // Get the list of supported modes on the server
        ConnectionManager.getInstance().getPort().getAvailableGameModes(serverModes);

        // Populate supported game modes list with modes that both this client and the server supports
        List<IGameModeFactory> clientModes = GameManager.getInstance().getSupportedGameModes();
        gameModeCombo.removeAllItems();

        for(int i = 0; i < serverModes.value.size(); i++) {
            // Search through the client mode list and see if we have a match
            for(int j = 0; j < clientModes.size(); j++) {
                if(serverModes.value.get(i).getId().equals(clientModes.get(j).getGameModeID())) {
                    // We have a match, add the mode
                    gameModeCombo.addItem(new ComboItem<String>(
                            clientModes.get(j).getGameModeName(),
                            clientModes.get(j).getGameModeID()));
                    break;
                }
            }
        }

        return true;
    }

    @Override
    public void setEnabled(boolean enabled) {
        super.setEnabled(enabled);

        // FIXME: Hack to disable the login window, so far this is the only card
        // we need to disable, so we'll do it manually here, java has no clean way to do this
        btnLogin.setEnabled(enabled);
        textUsername.setEnabled(enabled);
        jPasswordField1.setEnabled(enabled);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        loginPanel = new javax.swing.JPanel();
        btnLogin = new javax.swing.JButton();
        textUsername = new javax.swing.JTextField();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jPasswordField1 = new javax.swing.JPasswordField();
        mainPanel = new javax.swing.JPanel();
        startQuizButton = new javax.swing.JButton();
        editQuizzesButton = new javax.swing.JButton();
        quitButton = new javax.swing.JButton();
        startQuizPanel = new javax.swing.JPanel();
        jLabel3 = new javax.swing.JLabel();
        quizCombo = new javax.swing.JComboBox();
        jLabel4 = new javax.swing.JLabel();
        quizCodeText = new javax.swing.JTextField();
        btnStartQuiz = new javax.swing.JButton();
        jLabel5 = new javax.swing.JLabel();
        gameModeCombo = new javax.swing.JComboBox();
        gamePanel = new javax.swing.JPanel();

        setName("Form"); // NOI18N
        setLayout(new java.awt.CardLayout());

        loginPanel.setName("loginPanel"); // NOI18N
        loginPanel.setLayout(new java.awt.GridBagLayout());

        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(lecturequiz.teacherclient.LQTeacherClientApp.class).getContext().getResourceMap(MainMenu.class);
        btnLogin.setText(resourceMap.getString("btnLogin.text")); // NOI18N
        btnLogin.setName("btnLogin"); // NOI18N
        btnLogin.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnLoginActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.ipadx = 17;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(6, 12, 133, 156);
        loginPanel.add(btnLogin, gridBagConstraints);

        textUsername.setName("textUsername"); // NOI18N
        textUsername.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                textUsernameActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridheight = 2;
        gridBagConstraints.ipadx = 86;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(146, 12, 0, 156);
        loginPanel.add(textUsername, gridBagConstraints);

        jLabel1.setText(resourceMap.getString("jLabel1.text")); // NOI18N
        jLabel1.setName("jLabel1"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(148, 169, 0, 0);
        loginPanel.add(jLabel1, gridBagConstraints);

        jLabel2.setText(resourceMap.getString("jLabel2.text")); // NOI18N
        jLabel2.setName("jLabel2"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(14, 171, 0, 0);
        loginPanel.add(jLabel2, gridBagConstraints);

        jPasswordField1.setName("jPasswordField1"); // NOI18N
        jPasswordField1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jPasswordField1ActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.gridheight = 2;
        gridBagConstraints.ipadx = 86;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(12, 12, 0, 156);
        loginPanel.add(jPasswordField1, gridBagConstraints);

        add(loginPanel, "loginPanel");

        mainPanel.setName("mainPanel"); // NOI18N
        mainPanel.setLayout(new java.awt.GridBagLayout());

        startQuizButton.setText(resourceMap.getString("startQuizButton.text")); // NOI18N
        startQuizButton.setName("startQuizButton"); // NOI18N
        startQuizButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                startQuizButtonActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 153, 0, 154);
        mainPanel.add(startQuizButton, gridBagConstraints);

        editQuizzesButton.setText(resourceMap.getString("editQuizzesButton.text")); // NOI18N
        editQuizzesButton.setName("editQuizzesButton"); // NOI18N
        editQuizzesButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                editQuizzesButtonActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.ipadx = 11;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(10, 153, 10, 154);
        mainPanel.add(editQuizzesButton, gridBagConstraints);

        quitButton.setText(resourceMap.getString("quitButton.text")); // NOI18N
        quitButton.setName("quitButton"); // NOI18N
        quitButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                quitButtonActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.ipadx = 11;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 153, 0, 154);
        mainPanel.add(quitButton, gridBagConstraints);

        add(mainPanel, "mainPanel");

        startQuizPanel.setName("startQuizPanel"); // NOI18N
        startQuizPanel.setLayout(new java.awt.GridBagLayout());

        jLabel3.setText(resourceMap.getString("jLabel3.text")); // NOI18N
        jLabel3.setName("jLabel3"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        startQuizPanel.add(jLabel3, gridBagConstraints);

        quizCombo.setName("quizCombo"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridheight = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.ipadx = 120;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(0, 10, 0, 0);
        startQuizPanel.add(quizCombo, gridBagConstraints);

        jLabel4.setText(resourceMap.getString("jLabel4.text")); // NOI18N
        jLabel4.setName("jLabel4"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(10, 0, 0, 0);
        startQuizPanel.add(jLabel4, gridBagConstraints);

        quizCodeText.setText(resourceMap.getString("quizCodeText.text")); // NOI18N
        quizCodeText.setName("quizCodeText"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 4;
        gridBagConstraints.gridheight = 2;
        gridBagConstraints.ipadx = 66;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(10, 10, 0, 0);
        startQuizPanel.add(quizCodeText, gridBagConstraints);

        btnStartQuiz.setText(resourceMap.getString("btnStartQuiz.text")); // NOI18N
        btnStartQuiz.setName("btnStartQuiz"); // NOI18N
        btnStartQuiz.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnStartQuizActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 6;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(10, 10, 0, 0);
        startQuizPanel.add(btnStartQuiz, gridBagConstraints);

        jLabel5.setText(resourceMap.getString("jLabel5.text")); // NOI18N
        jLabel5.setName("jLabel5"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(10, 0, 0, 0);
        startQuizPanel.add(jLabel5, gridBagConstraints);

        gameModeCombo.setName("gameModeCombo"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        gridBagConstraints.gridheight = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(10, 10, 0, 0);
        startQuizPanel.add(gameModeCombo, gridBagConstraints);

        add(startQuizPanel, "startQuizPanel");

        gamePanel.setName("gamePanel"); // NOI18N
        add(gamePanel, "gamePanel");
    }// </editor-fold>//GEN-END:initComponents

    private void btnLoginActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnLoginActionPerformed
        LectureQuizService port = ConnectionManager.getInstance().getPort();

        switch(port.authenticate(textUsername.getText(), new String(jPasswordField1.getPassword()), true)) {
            case 0:
                setPanel(PANEL.MAIN_PANEL);
                break;

            case 1:
                JOptionPane.showMessageDialog(this, "Authentication failed");
                break;

            case 2:
                JOptionPane.showMessageDialog(this, "Username taken");
                break;
        }
}//GEN-LAST:event_btnLoginActionPerformed

    private void quitButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_quitButtonActionPerformed
        LQTeacherClientView.quit();
    }//GEN-LAST:event_quitButtonActionPerformed

    private void btnStartQuizActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnStartQuizActionPerformed
        //TODO: btnStartQuizActionPerformed: finish implementation

        // Make sure inputs are valid
        if(quizCodeText.getText().isEmpty()) {
            JOptionPane.showMessageDialog(this, "Quiz code not entered");
            return;
        }

        if(quizCombo.getSelectedItem() == null) {
            JOptionPane.showMessageDialog(this, "No quiz selected");
            return;
        }

        if(gameModeCombo.getSelectedItem() == null) {
            JOptionPane.showMessageDialog(this, "No game mode selected");
            return;
        }

        String gameMode = ((ComboItem<String>)gameModeCombo.getSelectedItem()).getId();
        int quizId = ((ComboItem<Integer>)quizCombo.getSelectedItem()).getId();
        String quizCode = quizCodeText.getText();

        // Try to start the selected quiz and game mode
        LectureQuizService port = ConnectionManager.getInstance().getPort();

        int ret = port.newQuiz(gameMode,quizId, quizCode,null);

        switch(ret) {
            case 0:
                break;

            case 4:
                JOptionPane.showMessageDialog(this, "Quiz code already in use by another game");
                return;

            case 5:
                JOptionPane.showMessageDialog(this, "Selected game mode is not supported by the server");
                return;

            case 6:
                JOptionPane.showMessageDialog(this, "Selected quiz could not be found on the server");
                return;

            default:
                JOptionPane.showMessageDialog(this, "Could not start quiz on the server");
                return;
        }

        // Create a game mode instance of the correct type
        try {
            AbstractGameMode game = GameManager.getInstance().createGameMode(gameMode,gamePanel);
            game.startQuiz(quizId, quizCode);
            setPanel(PANEL.GAME_PANEL);
        } catch(Exception e) {
            JOptionPane.showMessageDialog(this, "Could not instantiate a game.\r\n" + e.getMessage());
            port.endQuiz();
        }
    }//GEN-LAST:event_btnStartQuizActionPerformed

    private void startQuizButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_startQuizButtonActionPerformed
        setPanel(PANEL.START_QUIZ_PANEL);
    }//GEN-LAST:event_startQuizButtonActionPerformed

    private void jPasswordField1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jPasswordField1ActionPerformed
        btnLoginActionPerformed(evt);
    }//GEN-LAST:event_jPasswordField1ActionPerformed

    private void textUsernameActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_textUsernameActionPerformed
        btnLoginActionPerformed(evt);
    }//GEN-LAST:event_textUsernameActionPerformed

    private void editQuizzesButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_editQuizzesButtonActionPerformed
        setPanel(PANEL.EDIT_QUIZ_PANEL);
    }//GEN-LAST:event_editQuizzesButtonActionPerformed



    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnLogin;
    private javax.swing.JButton btnStartQuiz;
    private javax.swing.JButton editQuizzesButton;
    private javax.swing.JComboBox gameModeCombo;
    private javax.swing.JPanel gamePanel;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JPasswordField jPasswordField1;
    private javax.swing.JPanel loginPanel;
    private javax.swing.JPanel mainPanel;
    private javax.swing.JButton quitButton;
    private javax.swing.JTextField quizCodeText;
    private javax.swing.JComboBox quizCombo;
    private javax.swing.JButton startQuizButton;
    private javax.swing.JPanel startQuizPanel;
    private javax.swing.JTextField textUsername;
    // End of variables declaration//GEN-END:variables

}
