/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package lecturequiz.teacherclient.gamemodes;

import lecturequiz.teacherclient.gamemodes.utils.TextFlowRenderer;
import com.sun.opengl.util.FPSAnimator;
import com.sun.opengl.util.j2d.TextRenderer;
import com.sun.opengl.util.texture.Texture;
import com.sun.opengl.util.texture.TextureIO;
import java.awt.Color;
import java.awt.Font;
import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.IOException;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import javax.media.opengl.DebugGL;
import javax.media.opengl.GL;
import javax.media.opengl.GLAutoDrawable;
import javax.media.opengl.GLException;
import javax.media.opengl.glu.GLU;
import javax.swing.JOptionPane;
import lecturequiz.server.service.QuestionInfo;
import lecturequiz.server.service.StatisticsEntry;
import lecturequiz.server.service.GameStatus;
import lecturequiz.teacherclient.ConfigurationManager;

/**
 *
 * @author avenger
 */
public class PlainGameRenderer extends AbstractGameRenderer {

    /// The animator timer that makes sure the scene is rendered
    protected FPSAnimator animator;
    protected GLU glu;

    /**
     * The different states of this renderer
     */
    public enum STATE {
        RENDER_WELCOME,
        RENDER_QUESTION,
        RENDER_STATS,
        RENDER_OVERALL_STATS
    };

    /// Current state of the renderer
    protected STATE state;

    // Used to render the welcome screen
    protected GameStatus gameStatus;

    protected QuestionInfo question;
    protected Date questionStart;

    // Used to render statistics
    protected class Statistics {
        public Statistics() {
            answerDistribution = new HashMap<Integer, Integer>();
        }

        public HashMap<Integer,Integer> answerDistribution;
        public int correctAnswer;
        public int totalAnswers;
    };
    protected Statistics statistics;

    // Used to render the overall statistics at the end
    protected class OverallStatistics {
        public float correctAnswerRatio;
        public int questionCount;
        public String mostCorrectUser;
        public int mostCorrectCount;
    }
    protected OverallStatistics overallStatistics;

    // Different text renderers used by this renderer
    protected TextFlowRenderer headerRenderer;
    protected TextFlowRenderer questionRenderer;
    protected TextFlowRenderer[] answerRenderers;
    protected TextFlowRenderer mostCorrectUserRenderer;
    protected TextRenderer timerRenderer;
    protected TextRenderer statisticsRenderer;

    // The texture used for backround behind the different answers
    protected Texture answerBackground;

    protected float sceneHeight;
    protected float sceneWidth;

    private final int ANSWER_BOX_WIDTH = 400;
    private final int ANSWER_BOX_HEIGHT = 256;

    private final float SCENE_RESOLUTION_WIDTH = 1024.0f;
    private final float SCENE_RESOLUTION_HEIGHT = 768.0f;

    public PlainGameRenderer(int width, int height) {
        super(width,height);

        headerRenderer = new TextFlowRenderer(new Font("SansSerif", Font.BOLD, 36));
        timerRenderer = new TextRenderer(new Font("SansSerif", Font.BOLD, 28),
                ConfigurationManager.getInstance().getTextRendererUseMipMap());
        statisticsRenderer = new TextRenderer(new Font("SansSerif", Font.BOLD, 18),
                ConfigurationManager.getInstance().getTextRendererUseMipMap());
        mostCorrectUserRenderer = new TextFlowRenderer(new Font("SansSerif", Font.BOLD, 28));

        answerBackground = null;


        state = STATE.RENDER_WELCOME;
    }

    /**
     * Init the opengl draw area
     * @param drawable
     */
    public void init(GLAutoDrawable drawable) {
        GL gl = drawable.getGL();
        drawable.setGL(new DebugGL(gl));

        glu = new GLU();

        gl.glEnable(GL.GL_DEPTH_TEST);
        gl.glDepthFunc(GL.GL_LEQUAL);
        gl.glShadeModel(GL.GL_SMOOTH);
        gl.glHint(GL.GL_PERSPECTIVE_CORRECTION_HINT, GL.GL_NICEST);
        gl.glClearColor(0.0f, 0.0f, 0.0f, 1.0f);

        // Load textures
        try
        {
            answerBackground = TextureIO.newTexture(new File("data/AnswerBackground.png"), false);
        } catch (GLException ex) {
            JOptionPane.showMessageDialog(this, "Could not init texture: data/AnswerBackground.png\nGLException: " + ex.getMessage());
        } catch (IOException ex) {
            JOptionPane.showMessageDialog(this, "Could not init texture: data/AnswerBackground.png\nIOException: " + ex.getMessage());
        }

        animator = new FPSAnimator(this,30);
        animator.start();
    }

    /**
     * Called when  the drawing area changes. Makes sure the aspect ratio of the
     * rendering area is correct
     * @param drawable
     * @param x new X coordinate
     * @param y new Y coordinate
     * @param width new width
     * @param height new height
     */
    public void reshape(GLAutoDrawable drawable, int x, int y, int width, int height) {
        GL gl = drawable.getGL();

        // Recalculate the new 2D Scene width
        if((float)width / SCENE_RESOLUTION_WIDTH < (float)height / SCENE_RESOLUTION_HEIGHT) {
            sceneWidth = SCENE_RESOLUTION_WIDTH;
            sceneHeight = ((float)height / ((float)width / SCENE_RESOLUTION_WIDTH));
        } else {
            sceneHeight = SCENE_RESOLUTION_HEIGHT;
            sceneWidth = ((float)width / ((float)height / SCENE_RESOLUTION_HEIGHT));
        }

        gl.glViewport(0,0,width,height);
    }

    /**
     * Called when a frame needs to be rendered
     * @param drawable
     */
    public void display(GLAutoDrawable drawable) {
        GL gl = drawable.getGL();
        gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);
        
        switch(state) {
            case RENDER_WELCOME:
                renderWelcome(drawable);
                break;

            case RENDER_QUESTION:
                renderQuestion(drawable);
                break;

            case RENDER_STATS:
                renderStats(drawable);
                break;

            case RENDER_OVERALL_STATS:
                renderOverallStats(drawable);
                break;
        }
    }

    /**
     * Called when the display is changed
     * @param drawable
     * @param modeChanged true if the mode has changed
     * @param deviceChanged true if the device has changed
     */
    public void displayChanged(GLAutoDrawable glad, boolean bln, boolean bln1) {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    /**
     * Sets the state to the render welcome screen state and renderes it with
     * the supplied game status
     * @param gameStatus The status of the game to show on the welcome screen
     */
    public void showWelcomeScreen(GameStatus gameStatus) {
        this.gameStatus = gameStatus;
        state = STATE.RENDER_WELCOME;
    }

    /**
     * Sets this class in the "render question" state, and sets the current
     * question to be rendered
     * @param question the question to be rendered
     */
    public void showQuestion(QuestionInfo question) {
        this.question = question;
        questionStart = new Date();

        // Make sure we reflow the text so
        int fontSize = 36;
        do {
            questionRenderer = new TextFlowRenderer(new Font("SansSerif", Font.BOLD, fontSize));
            questionRenderer.setText(question.getText(), 950);
            fontSize -= 4;
        } while(questionRenderer.getHeight() > 130 && fontSize > 10);

        answerRenderers = new TextFlowRenderer[question.getAnswers().size()];
        for(int i = 0; i < question.getAnswers().size(); i++) {
            char[] caption = { (char)('A' + i) };
            String alternative = new String(caption);
            answerRenderers[i] = new TextFlowRenderer(new Font("SansSerif", Font.PLAIN, 24));
            answerRenderers[i].setText(alternative + ": " + question.getAnswers().get(i).getText(), ANSWER_BOX_WIDTH - 20);
        }

        state = STATE.RENDER_QUESTION;
    }

    /**
     * Switches to render statistics from the last question
     */
    public void showStatistics(List<StatisticsEntry> stats) {
        // Parse the response
        statistics = new Statistics();

        for(int i = 0; i < stats.size(); i++) {
            try {
                if(stats.get(i).getKey().equals("correctAnswer")) {
                    statistics.correctAnswer = Integer.parseInt(stats.get(i).getValue());
                } else {
                    statistics.answerDistribution.put(Integer.parseInt(stats.get(i).getKey()),
                            Integer.parseInt(stats.get(i).getValue()));
                    statistics.totalAnswers += Integer.parseInt(stats.get(i).getValue());
                }
            } catch(NumberFormatException e) {
                System.out.println("Unknown statistics field:" + stats.get(i).getKey() +
                        " with value " + stats.get(i).getValue());
            }
        }

        state = STATE.RENDER_STATS;
    }

    /**
     * Switches to render the overall statistics of the entire quiz
     * @param stats
     */
    public void showOverallStatistics(List<StatisticsEntry> stats) {

        overallStatistics = new OverallStatistics();

        // Loop though the stats and pick out the values we need
        for(int i = 0; i < stats.size(); i++) {
            if(stats.get(i).getKey().equals("correct")) {
                overallStatistics.correctAnswerRatio = Float.parseFloat(stats.get(i).getValue()) * 100;
            } else if(stats.get(i).getKey().equals("questionCount")) {
                overallStatistics.questionCount = Integer.parseInt(stats.get(i).getValue());
            } else if(stats.get(i).getKey().equals("mostCorrectUser")) {
                overallStatistics.mostCorrectUser = stats.get(i).getValue();
            } else if(stats.get(i).getKey().equals("mostCorrectCount")) {
                overallStatistics.mostCorrectCount = Integer.parseInt(stats.get(i).getValue());
            }
        }

        // Reflow the textrenderer
        String mostCorrectUserTxt = "The user with most correct answers was: " + overallStatistics.mostCorrectUser + " with " +
                Integer.toString(overallStatistics.mostCorrectCount) + " of " + Integer.toString(overallStatistics.questionCount) +
                " correct answers";

        mostCorrectUserRenderer.setText(mostCorrectUserTxt, 800);

        state = STATE.RENDER_OVERALL_STATS;
    }

    /**
     * A empty cleanup method as we need no special cleanup
     */
    public void cleanup()
    {

    }

    /**
     * Render the welcome screen of this quiz
     * @param drawable the GLAutoDrawable-object to do the drawing on
     */
    protected void renderWelcome(GLAutoDrawable drawable) {
        GL gl = drawable.getGL();

        // Change to projection matrix.
        gl.glMatrixMode(GL.GL_PROJECTION);
        gl.glLoadIdentity();

        // Use a orthographic view
        gl.glOrtho(-(sceneWidth/2), sceneWidth/2, -(sceneHeight/2), sceneHeight/2, 0, 1);

        // Change back to model view matrix.
        gl.glMatrixMode(GL.GL_MODELVIEW);
        gl.glLoadIdentity();

        gl.glEnable(GL.GL_TEXTURE_2D);
        gl.glDisable(GL.GL_DEPTH_TEST);

        // Reuse the timer-render as it has the most proper font
        String quizCodeTxt = "Quiz code: " + gameStatus.getQuizCode();
        Rectangle2D playerCountPos = sceneCoordToTextCoord(-(SCENE_RESOLUTION_WIDTH/2) + 15, (SCENE_RESOLUTION_HEIGHT/2) - 45);
        Rectangle2D quizCodePos = sceneCoordToTextCoord(-((int)timerRenderer.getBounds(quizCodeTxt).getWidth()/2), -300);
        timerRenderer.beginRendering((int)sceneWidth, (int)sceneHeight);
        timerRenderer.draw("Joined players: " + gameStatus.getPlayerCount().get(0),
                (int)playerCountPos.getWidth(), (int)playerCountPos.getHeight());
        timerRenderer.draw(quizCodeTxt,(int)quizCodePos.getWidth(),(int)quizCodePos.getHeight());
        timerRenderer.endRendering();
    }

    /**
     * Renders a question on the screen
     * @param drawable the GLAutoDrawable-object to do the drawing on
     */
    protected void renderQuestion(GLAutoDrawable drawable) {
        GL gl = drawable.getGL();

        // Change to projection matrix.
        gl.glMatrixMode(GL.GL_PROJECTION);
        gl.glLoadIdentity();

        // Use a orthographic view
        gl.glOrtho(-(sceneWidth/2), sceneWidth/2, -(sceneHeight/2), sceneHeight/2, 0, 1);

        // Change back to model view matrix.
        gl.glMatrixMode(GL.GL_MODELVIEW);
        gl.glLoadIdentity();

        gl.glEnable(GL.GL_TEXTURE_2D);
        gl.glDisable(GL.GL_DEPTH_TEST);

        // Render boxes for the answers, 4 answers, 4 boxes
        if(answerBackground != null)
            answerBackground.bind();

        gl.glBegin(GL.GL_TRIANGLES);

        // TODO: Replace this with calculated values to support more answers
        int[] startXPoints = {-412, -412 + ANSWER_BOX_WIDTH + 50, -412, -412 + ANSWER_BOX_WIDTH + 50};
        int[] startYPoints = {200, 200, 200 - ANSWER_BOX_HEIGHT -50, 200 - ANSWER_BOX_HEIGHT - 50};

        for(int i = 0; i < 4; i++) {
            gl.glColor3fv(getAnswerColor(i), 0);

            gl.glTexCoord2f(0.0f, 0.0f);
            gl.glVertex2i(startXPoints[i],startYPoints[i]);
            gl.glTexCoord2f(1.0f, 0.0f);
            gl.glVertex2i(startXPoints[i] + ANSWER_BOX_WIDTH, startYPoints[i]);
            gl.glTexCoord2f(1.0f, 1.0f);
            gl.glVertex2i(startXPoints[i] + ANSWER_BOX_WIDTH,startYPoints[i] - ANSWER_BOX_HEIGHT);

            gl.glTexCoord2f(0.0f, 0.0f);
            gl.glVertex2i(startXPoints[i],startYPoints[i]);
            gl.glTexCoord2f(1.0f, 1.0f);
            gl.glVertex2i(startXPoints[i] + ANSWER_BOX_WIDTH, startYPoints[i] - ANSWER_BOX_HEIGHT);
            gl.glTexCoord2f(0.0f, 1.0f);
            gl.glVertex2i(startXPoints[i], startYPoints[i] - ANSWER_BOX_HEIGHT);
        }
        gl.glEnd();

        // Draw the question
        Rectangle2D textPos = sceneCoordToTextCoord((-(questionRenderer.getWidth() / 2)), 346);
        questionRenderer.beginRendering((int)sceneWidth,(int)sceneHeight);
        questionRenderer.setColor(1.0f,1.0f,1.0f,1.0f);
        questionRenderer.draw((int)textPos.getWidth(), (int)textPos.getHeight());
        questionRenderer.endRendering();

        // Calculate the time left of the question and draw it
        int timeLeft = question.getTimeout() - ((int)((new Date().getTime() - questionStart.getTime()) / 1000));

        if(timeLeft < 0)
            timeLeft = 0;
        
        String timeLeftStr = "Time left: " + Integer.toString(timeLeft);

        textPos = sceneCoordToTextCoord(- (float)(timerRenderer.getBounds(timeLeftStr).getWidth() / 2),
                316.0f - (float)(questionRenderer.getHeight() + 5));

        timerRenderer.beginRendering((int)sceneWidth,(int)sceneHeight);
        timerRenderer.draw(timeLeftStr,(int)textPos.getWidth(), (int)textPos.getHeight());
        timerRenderer.endRendering();

        // Split the answers up in two columns and render them
        for(int i = 0; i < question.getAnswers().size() && i < 4; i++) {
            textPos = sceneCoordToTextCoord(startXPoints[i] + ((ANSWER_BOX_WIDTH + 10) / 2) - ((int)answerRenderers[i].getWidth() / 2),
                    startYPoints[i] - ((ANSWER_BOX_HEIGHT + 10) / 2) + ((int)answerRenderers[i].getHeight() / 2));

            answerRenderers[i].beginRendering((int)sceneWidth,(int)sceneHeight);
            answerRenderers[i].setColor(1.0f, 1.0f, 1.0f, 1.0f);
            answerRenderers[i].draw((int)textPos.getWidth(), (int)textPos.getHeight());
            answerRenderers[i].endRendering();
        }

        gl.glEnable(GL.GL_DEPTH_TEST);
        gl.glDisable(GL.GL_TEXTURE_2D);
    }

    /**
     * Renderes the statistics after a question has timed out
     * @param drawable the drawable to render on
     */
    protected void renderStats(GLAutoDrawable drawable) {
        GL gl = drawable.getGL();

        // Change to projection matrix.
        gl.glMatrixMode(GL.GL_PROJECTION);
        gl.glLoadIdentity();

        // Use a orthographic view
        gl.glOrtho(-(sceneWidth/2), sceneWidth/2, -(sceneHeight/2), sceneHeight/2, 0, 1);

        // Change back to model view matrix.
        gl.glMatrixMode(GL.GL_MODELVIEW);
        gl.glLoadIdentity();


        // Simple histogram of resulsts
        float xLoc = -500.0f;
        for(int i = 0; i < question.getAnswers().size(); i++) {
            // Calculate the percentage of answers
            int id = question.getAnswers().get(i).getId();
            Integer count = statistics.answerDistribution.get(id);
            if(count == null)
                count = 0;

            float percentage = 0.0f;
            if(statistics.totalAnswers != 0)
                percentage = ((float)count / statistics.totalAnswers);

            float height = percentage * 500.0f;
            if(height < 1.0f)
                height = 1.0f;

            float[] color = getAnswerColor(i);
            drawBox(xLoc, -300.0f, 0.0f, 75.0f, height, color[0], color[1], color[2], gl);

            // Draw text under the bars
            Rectangle2D coord = sceneCoordToTextCoord(xLoc, -300.0f);
            String txt = String.format("%.2f%%",percentage * 100.0f);
            statisticsRenderer.beginRendering((int)sceneWidth, (int)sceneHeight);
            Rectangle2D bounds = statisticsRenderer.getBounds(txt);
            if(id == statistics.correctAnswer)
                statisticsRenderer.setColor(Color.GREEN);
            else
                statisticsRenderer.setColor(Color.RED);
            statisticsRenderer.draw(txt,
                    (int)(coord.getWidth() + 37.5f - (bounds.getWidth() / 2)),
                    (int)(coord.getHeight() - bounds.getHeight() - 2.0f));

            // Draw answer-captions
            statisticsRenderer.setColor(Color.WHITE);
            coord = sceneCoordToTextCoord(xLoc, -300 + height);
            char[] caption = { (char)('A' + i) };
            txt = new String(caption);
            bounds = statisticsRenderer.getBounds(txt);
            statisticsRenderer.draw(txt,
                    (int)(coord.getWidth() + 37.5f - (bounds.getWidth() / 2)),
                    (int)(coord.getHeight() + 3));
            statisticsRenderer.endRendering();

            xLoc += 100; // Advance the bar-position
        }


        // Find the widest answer string and adjust after that
        int widestString = 0;
        for(int i = 0; i < question.getAnswers().size(); i++) {
            if(answerRenderers[i].getWidth() > widestString)
                widestString = (int)answerRenderers[i].getWidth();
        }


        // Render the answer alternatives on the right
        int answerPosY = 250;
        for(int i = 0; i < question.getAnswers().size(); i++) {

            Rectangle2D ansPos = sceneCoordToTextCoord((SCENE_RESOLUTION_WIDTH/2) - 32 - widestString, answerPosY);
            answerRenderers[i].beginRendering((int)sceneWidth, (int)sceneHeight);
            if(question.getAnswers().get(i).getId() == statistics.correctAnswer)
                answerRenderers[i].setColor(Color.GREEN);
            else
                answerRenderers[i].setColor(Color.RED);
            answerRenderers[i].draw((int)ansPos.getWidth(), (int)ansPos.getHeight());
            answerRenderers[i].endRendering();

            answerPosY -= answerRenderers[i].getHeight() + 36;
        }


        // Render the text on top

        String txt = "Statistics:";
        Rectangle2D textPos = sceneCoordToTextCoord((-((float)headerRenderer.getBounds(txt).getWidth() / 2)), 316);
        headerRenderer.beginRendering((int)sceneWidth, (int)sceneHeight);
        headerRenderer.setColor(1.0f,1.0f,1.0f,1.0f);
        headerRenderer.draw(txt,(int)textPos.getWidth(), (int)textPos.getHeight());
        headerRenderer.endRendering();
    }

    /**
     * Renders the overall stats
     * @param drawable the drawable to draw on
     */
    protected void renderOverallStats(GLAutoDrawable drawable) {
        GL gl = drawable.getGL();

        // Change to projection matrix.
        gl.glMatrixMode(GL.GL_PROJECTION);
        gl.glLoadIdentity();

        // Use a orthographic view
        gl.glOrtho(-(sceneWidth/2), sceneWidth/2, -(sceneHeight/2), sceneHeight/2, 0, 1);

        // Change back to model view matrix.
        gl.glMatrixMode(GL.GL_MODELVIEW);
        gl.glLoadIdentity();

        String headerTxt = "Overall statistics:";
        String correctRatioTxt = String.format("%.2f%% of all answers were correct",overallStatistics.correctAnswerRatio);
        Rectangle2D headerTxtPos = sceneCoordToTextCoord(-((float)headerRenderer.getBounds(headerTxt).getWidth() / 2), 316);
        Rectangle2D correctRatioPos = sceneCoordToTextCoord(-((float)headerRenderer.getBounds(correctRatioTxt).getWidth() / 2), 200);
        headerRenderer.beginRendering((int)sceneWidth, (int)sceneHeight);
        headerRenderer.setColor(1.0f,1.0f,1.0f,1.0f);
        headerRenderer.draw(headerTxt,(int)headerTxtPos.getWidth(), (int)headerTxtPos.getHeight());
        headerRenderer.draw(correctRatioTxt,(int)correctRatioPos.getWidth(),(int)correctRatioPos.getHeight());
        headerRenderer.endRendering();

        if(overallStatistics.mostCorrectUser != null) {
            Rectangle2D mostCorrectPos = sceneCoordToTextCoord(-(mostCorrectUserRenderer.getWidth() / 2), 0);
            mostCorrectUserRenderer.beginRendering((int)sceneWidth, (int)sceneHeight);
            mostCorrectUserRenderer.draw((int)mostCorrectPos.getWidth(),(int)mostCorrectPos.getHeight());
            mostCorrectUserRenderer.endRendering();
        }
    }

    /**
     * Helper function to convert scene coordinates to the coordinates the text
     * renderer uses
     * @param x
     * @param y
     * @param sceneWidth
     * @param sceneHeight
     * @return
     */
    protected Rectangle2D sceneCoordToTextCoord(float x, float y) {
        Rectangle2D rect = new Rectangle2D.Float();
        rect.setRect(0,0,x + (sceneWidth/2), y + (sceneHeight/2));
        return rect;
    }

    /**
     * Helper function to allow for easy editing of colors used and the number
     * of answer alternatives
     * @param nr The index of the answer. Starts at 0.
     * @return An array of floats with RGB values, white if the index is invalid
     */
    protected float[] getAnswerColor(int nr) {
        float[][] colors = { {1.0f, 0.0f, 0.0f}, {0.0f, 0.33f, 0.0f}, {0.0f, 0.0f, 1.0f}, {1.0f, 0.5f, 0.0f} };

        if(nr < 4 && nr >= 0) {
            return colors[nr];
        }

        float[] ret = {1.0f, 1.0f, 1.0f};
        return ret;
    }

    /**
     * Helperfunction for drawing a box
     * @param x x-coordinate of the bottom left front corner
     * @param y y-coordinate of the bottom left front corner
     * @param z z-coordinate of the bottom left front corner
     * @param width Width of the square bottom of the box
     * @param height Height of the box
     * @param r red color component
     * @param g green color component
     * @param b blue color component
     * @param gl GL object to access drawing functions
     */
    protected void drawBox(float x, float y, float z, float width, float height,
            float r, float g, float b,GL gl) {
        gl.glBegin(GL.GL_TRIANGLES);
        gl.glColor3f(r,g,b);
        
        gl.glVertex3f(x,y,z);
        gl.glVertex3f(x + width,y + height,z);
        gl.glVertex3f(x + width,y,z);
        gl.glVertex3f(x,y,z);
        gl.glVertex3f(x,y + height,z);
        gl.glVertex3f(x + width,y + height,z);

        gl.glEnd();
    }
}
