/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package lecturequiz.teacherclient.gamemodes.utils;

import com.sun.opengl.util.j2d.TextRenderer;
import java.awt.Font;
import java.awt.font.FontRenderContext;
import java.awt.font.LineBreakMeasurer;
import java.awt.font.TextAttribute;
import java.awt.geom.Rectangle2D;
import java.text.AttributedString;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import lecturequiz.teacherclient.ConfigurationManager;

/**
 * Helper-class to render text in a flow
 * @author knutant
 */
public class TextFlowRenderer extends TextRenderer {
    private ArrayList<String> lines;
    private String text;
    private float width;
    private float height;
    private int lineSpacing;


    /**
     * Constructor
     * @param font The font used when rendering all the text
     */
    public TextFlowRenderer(Font font) {
        super(font,ConfigurationManager.getInstance().getTextRendererUseMipMap());

        lines = new ArrayList<String>();
    }

    /**
     * Sets the text and its width to render
     * @param text the text to render
     * @param width the max width of the text
     */
    public void setText(String text, float width) {
        this.text = text;
        reflow(width);

        if(lines.size() > 1)
            this.width = width;
        else
            this.width = (float)this.getBounds(lines.get(0)).getWidth();
    }

    /**
     * Sets the max width the text is allowed to use when rendering
     * @param width new width of the text area
     */
    public void setWidth(float width) {
        this.width = width;
        reflow(width);
    }

    /**
     * Return the text this object is rendering
     * @return The text rendered, without added line breaks
     */
    public String getText() {
        return text;
    }

    /**
     * Return the width of the text
     * @return width of the text
     */
    public float getWidth() {
        return width;
    }

    /**
     * Return the hight of the text area
     * @return height of all the lines
     */
    public float getHeight() {
        return height;
    }

    /**
     * Draw the text. Starting from a high Y-value and going downward
     * @param x X coordinate to start and draw from
     * @param y Y  coordinate to start and draw from
     */
    public void draw(int x, int y) {
        for(int i = 0; i < lines.size(); i++) {
            super.draw(lines.get(i), x, y);
            y -= lineSpacing;
        }
    }

    /**
     * Reflow the text to a specific width, by splitting it up in lines
     * @param width Maximum width of a line
     */
    private void reflow(float width) {
        lineSpacing = 0;
        int numLines = 0;
        lines.clear();

        FontRenderContext frc = super.getFontRenderContext();
        Map attrs = new HashMap();
        attrs.put(TextAttribute.FONT, super.getFont());
        AttributedString str = new AttributedString(text, attrs);
        LineBreakMeasurer measurer = new LineBreakMeasurer(str.getIterator(), frc);
        int curPos = 0;
        while (measurer.getPosition() < text.length()) {
            int nextPos = measurer.nextOffset(width);
            String line = text.substring(curPos, nextPos);
            Rectangle2D bounds = super.getBounds(line);
            lines.add(line);
            lineSpacing += (int) bounds.getHeight();
            ++numLines;
            curPos = nextPos;
            measurer.setPosition(curPos);
        }

        height = (float)lineSpacing;
        lineSpacing = (int) ((float) lineSpacing / (float) numLines) + 5;
    }
}
