/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package lecturequiz.teacherclient.service;

import java.net.MalformedURLException;
import java.net.URL;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.Holder;
import javax.xml.ws.WebServiceException;
import lecturequiz.server.service.LectureQuizService;
import lecturequiz.server.service.LectureQuizServiceService;
import lecturequiz.teacherclient.ConfigurationManager;

/**
 * Class to mange the connection to the web service.
 * This class basicly creates a new instance of the web service port
 * @author Knut A. Tidemann
 */
public class ConnectionManager {
    private LectureQuizService port;
    private LectureQuizServiceService service;

    private static ConnectionManager _instance = null;

    private ConnectionManager() {
        // Init the connection to the SOAP service
        // TODO: Get URL from config file
        URL wsdl = null;
        try
        {
            wsdl = new URL(ConfigurationManager.getInstance().getWebServiceURL());
        } catch(MalformedURLException ex) {
            // Since we're doing a connection check later, we'll just leave it at null
            wsdl = null;
        }

        service = new LectureQuizServiceService(wsdl);
        port = service.getLectureQuizServicePort();

        ((BindingProvider)port).getRequestContext().put(BindingProvider.SESSION_MAINTAIN_PROPERTY,true);
    }

    /**
     * Get the singleton instance of this class
     * @return ConnectionManager instance
     */
    public static synchronized ConnectionManager getInstance() {
        if(_instance == null)
            _instance = new ConnectionManager();

        return _instance;
    }

    /**
     * Destroys the previous instance and creates a new one.
     * Used when you want to reconnect to the web service.
     * @return A reference to the new instance.
     */
    public static ConnectionManager createNewInstance() {
        _instance = new ConnectionManager();

        return _instance;
    }

    /**
     * Returns the port to the lecture quiz web service
     * @return LectureQuizService port used for method calls.
     */
    public LectureQuizService getPort() {
        return port;
    }

    /**
     * Tests a connection to the given url. Will thrown an exception on error.
     * @param url URL to the web service
     * @return The version string of the service if a conncetion could be made
     * @throws MalformedURLException If the URL was malformed.
     * @throws WebServiceException If the connection attempt failed.
     */
    public static String testConnection(String url) throws MalformedURLException, WebServiceException {
        URL wsdl = new URL(url);


        LectureQuizServiceService testservice = new LectureQuizServiceService(wsdl);
        LectureQuizService testport = testservice.getLectureQuizServicePort();

        Holder<String> serviceVersion = new Holder<String>();

        testport.getServiceVersion(serviceVersion);

        if(serviceVersion.value == null)
            throw new WebServiceException("Could not connect to server");

        return serviceVersion.value;
    }
}
