/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.ntnu.idi.lqwebclient.client.gamemode;

import com.google.gwt.event.shared.HandlerManager;
import java.util.ArrayList;
import java.util.List;
import org.ntnu.idi.lqwebclient.client.presenter.AbstractGameModePresenter;
import org.ntnu.idi.lqwebclient.client.service.LQserviceAsync;



/**
 * A game manager that keeps track of all supported game modes.
 *
 * @author avenger
 */
public class GameManager {
    // Singleton instance
    private static GameManager _instance = null;

    // List of supported gametypes
    private ArrayList<IGameModeFactory> supportedModes;

    // The instance of the current game that is running
    private AbstractGameModePresenter currentGame;

    /**
     * Private singleton constructor
     */
    private GameManager() {
        supportedModes = new ArrayList<IGameModeFactory>();

        //Add all supported game modes here!
        supportedModes.add(new PlainGameFactory());

        currentGame = null;
    }

    /**
     * Returns the singleton instance of the GameFactory class
     * @return GameFactory singleton instance.
     */
    public static synchronized GameManager getInstance() {
        if(_instance == null)
            _instance = new GameManager();

        return _instance;
    }

    /**
     * Checks if a game mode is supported by this client.
     * @param gameMode ID of the game mode to check.
     * @return True if the game mode is supported, false otherwise.
     */
    public boolean isGameModeSupported(String gameMode) {
        for(int i = 0; i < supportedModes.size(); i++)
            if(supportedModes.get(i).getGameModeID().equals(gameMode))
                return true;

        return false;
    }

    /**
     * Creates a game mode instance from the desired game mode string id and sets
     * it as the current game.
     * @param gameMode The ID of the game mode to create
     * @return An instance of the correct game mode class. null if the id is not found
     */
    public AbstractGameModePresenter createGameMode(String gameMode, LQserviceAsync rpcService, HandlerManager eventBus) {
        for(int i = 0; i < supportedModes.size(); i++) {
            if(supportedModes.get(i).getGameModeID().equals(gameMode)) {
                currentGame = supportedModes.get(i).createInstance(rpcService, eventBus);
                return currentGame;
            }
        }

        return null;
    }


    /**
     * Returns the list of supported game modes
     * @return final reference to the list of supported game modes
     */
    public final List<IGameModeFactory> getSupportedGameModes() {
        return (List)supportedModes;
    }

    /**
     * Returns the current running game, or null if no game is running
     * @return AbstractGameMode instance of the current game, null if no game is running
     */
    public AbstractGameModePresenter getCurrentGame() {
        return currentGame;
    }
}
