/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.ntnu.idi.lqwebclient.client.presenter;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.logical.shared.CloseEvent;
import com.google.gwt.event.logical.shared.CloseHandler;
import com.google.gwt.event.shared.HandlerManager;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.FocusWidget;
import com.google.gwt.user.client.ui.HasValue;
import com.google.gwt.user.client.ui.HasWidgets;
import com.google.gwt.user.client.ui.Widget;
import org.ntnu.idi.lqwebclient.client.gamemode.GameManager;
import org.ntnu.idi.lqwebclient.client.holder.UserHolder;
import org.ntnu.idi.lqwebclient.client.service.LQserviceAsync;


/**
 * A Presenter handling the logon sequence, and starting a new GameModePresenter when the logon is successful
 * @author erlingbo
 */
public class LogInPresenter implements Presenter{

    /**
     * The display used by this Presenter. The display is implementet by one or more views (eg. PC and mobile)
     */
    public interface Display {
        public final int LOGIN_STATE    = 1;
        public final int JOINQUIZ_STATE = 2;
        FocusWidget getLogInButton();
        HasValue<String> getUsername();
        HasValue<String> getQuizCode();
        Widget asWidget();
        void displayError(String errorCaption, String errorMessage, CloseHandler handler);
        public void setState(int state);
        public int getState();
    }
    private final LQserviceAsync rpcService;
    private final HandlerManager eventBus;
    private final Display display;
    protected HasWidgets container;
    private String gamemodeString = "";

    public LogInPresenter(LQserviceAsync rpcService, HandlerManager eventBus, Display view) {
        this.rpcService = rpcService;
        this.eventBus = eventBus;
        this.display = view;
    }
    
    public void go(HasWidgets container) {


        

        //TODO: sjekke om brukeren er logget inn fra før

        this.container = container;
        rpcService.getUser(getUserCallback);

    }

    /**
     * RPC callback for the getUser function
     */
    final AsyncCallback<UserHolder> getUserCallback = new AsyncCallback<UserHolder>(){

        public void onFailure(Throwable caught) {
            display.setState(display.LOGIN_STATE);
            bind();
        }

        public void onSuccess(UserHolder result) {
            if (result == null){
                display.setState(display.LOGIN_STATE);
            }
            else {
                display.setState(display.JOINQUIZ_STATE);
                display.getUsername().setValue(result.getUsername());
            }
            bind();
        }
    };

    private void bind() {
        this.container.clear();
        this.container.add(display.asWidget());
        display.getLogInButton().addClickHandler(new ClickHandler() {

            public void onClick(ClickEvent event) {
                String username = display.getUsername().getValue();
                display.getLogInButton().setEnabled(false);
                rpcService.authenticate(username, authCallback);
            }
        });
    }

     /**
     * RPC callback for the authenticate function
     */
    final AsyncCallback<String> authCallback = new AsyncCallback<String>(){

        public void onFailure(Throwable caught) {
            display.displayError("Login Failed", caught.getMessage(), basicCloseHandler);
        }

        public void onSuccess(String result) {
            //Checks if the game mode on the server is supported by this client
            rpcService.isGamemodeCompatible(display.getQuizCode().getValue(), isCompatibleCallback);

        }

    };

    /**
     * RPC callback for the isGamemodeCompatible function
     */
    final AsyncCallback<String> isCompatibleCallback = new AsyncCallback<String>(){

        public void onFailure(Throwable caught) {
            display.displayError("GameMode error", caught.getMessage(), basicCloseHandler);
        }

        public void onSuccess(String result) {
            gamemodeString = result;
            rpcService.joinQuiz(display.getQuizCode().getValue(), joinQuizCallback);
        }

    };

    /**
     * RPC callback for the joinQuiz function
     */
    final AsyncCallback<Boolean> joinQuizCallback = new AsyncCallback<Boolean>(){

        public void onFailure(Throwable caught) {
            display.displayError("JoinQuiz error", caught.getMessage(), basicCloseHandler);
        }

        public void onSuccess(Boolean result) {

            //starts the appropriate GameModePresenter
            Presenter presenter = GameManager.getInstance().createGameMode(gamemodeString, rpcService, eventBus);
            presenter.go(container);
        }
        
    };

    /**
     * Basic CloseHandler for the error dialog of the display view
     */
    final CloseHandler basicCloseHandler = new CloseHandler() {
        public void onClose(CloseEvent event) {
                    display.getLogInButton().setEnabled(true);
                }
    };

}
