/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.ntnu.idi.lqwebclient.client.presenter;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.logical.shared.CloseEvent;
import com.google.gwt.event.logical.shared.CloseHandler;
import com.google.gwt.event.shared.HandlerManager;
import com.google.gwt.i18n.client.NumberFormat;
import com.google.gwt.user.client.Timer;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.FocusWidget;
import com.google.gwt.user.client.ui.HasText;
import com.google.gwt.user.client.ui.HasWidgets;
import com.google.gwt.user.client.ui.Widget;
import java.util.List;
import org.ntnu.idi.lqwebclient.client.exception.AnswerException;
import org.ntnu.idi.lqwebclient.client.exception.EndOfQuizException;
import org.ntnu.idi.lqwebclient.client.holder.QuestionHolder;
import org.ntnu.idi.lqwebclient.client.holder.StatisticsHolder;
import org.ntnu.idi.lqwebclient.client.service.LQserviceAsync;
import org.ntnu.idi.lqwebclient.client.view.PlainGameView;

/**
 *
 * @author erlingbo
 */
public class PlainGamePresenter extends AbstractGameModePresenter{

    protected final int REFRESH_RATE = 2000;
    protected final Display display;
    protected HasWidgets container;
    protected int answerAid;
    protected int answerBid;
    protected int answerCid;
    protected int answerDid;
    protected Timer questionTimer;
    protected Timer statisticsTimer;
    protected Timer submitTimer;
    protected int lastAnswer;

    /**
     * The display used by this Presenter. The display is implementet by one or more views (eg. PC and mobile)
     */
    public interface Display {
        public final int LOADING_STATE = 1;
        public final int QUESTION_STATE = 2;
        public final int STATISTICS_STATE = 3;
        public final int STATISTICS_END_STATE = 4;
        public final int STATISTICS_OVERALL_STATE = 5;
        public final int STATISTICS_GOOD = 11;
        public final int STATISTICS_BAD = 12;
        public void setState(int state);
        public FocusWidget getAnswerAButton();
        public FocusWidget getAnswerBButton();
        public FocusWidget getAnswerCButton();
        public FocusWidget getAnswerDButton();
        public FocusWidget getEndStatisticsButton();
        public HasText getQuestionText();
        public void setLoadingText(String text);
        public void setStatistics(String text, int type);
        public Widget asWidget();
        void displayError(String errorCaption, String errorMessage, CloseHandler handler);
    }

    public PlainGamePresenter(LQserviceAsync rpcService, HandlerManager eventBus) {
        super(rpcService, eventBus);
        this.display = new PlainGameView();
    }

    @Override
    public void go(HasWidgets container) {
        bind();
        rpcService.getCurrentQuestion(getCurrentQuestionCallback);
        questionTimer = new Timer() {
            public void run(){
                
                rpcService.getCurrentQuestion(getCurrentQuestionCallback);
            }
        };
        questionTimer.scheduleRepeating(REFRESH_RATE);
        statisticsTimer = new Timer() {
            public void run(){
                rpcService.getQuestionStatistics(getQuestionStatisticsCallback);
            }
        };
        submitTimer = new Timer() {
            public void run(){
                answerWasLate();
            }
        };
        this.container = container;
        this.container.clear();

        display.setState(display.LOADING_STATE);
        display.setLoadingText("Waiting for the Quiz Master to start the quiz.");
        
        this.container.add(display.asWidget());
    }

    private void bind() {
        display.getAnswerAButton().addClickHandler(new ClickHandler() {public void onClick(ClickEvent event) {submitAnswer(answerAid);}});
        display.getAnswerBButton().addClickHandler(new ClickHandler() {public void onClick(ClickEvent event) {submitAnswer(answerBid);}});
        display.getAnswerCButton().addClickHandler(new ClickHandler() {public void onClick(ClickEvent event) {submitAnswer(answerCid);}});
        display.getAnswerDButton().addClickHandler(new ClickHandler() {public void onClick(ClickEvent event) {submitAnswer(answerDid);}});
        display.getEndStatisticsButton().addClickHandler(new ClickHandler() {public void onClick(ClickEvent event) {
                statisticsTimer.cancel();
                questionTimer.cancel();
                rpcService.getOverallStatistics(getOverallStatisticsCallback);
//                display.setStatistics("End of Quiz. Thanks for playing", display.STATISTICS_GOOD);
//                display.setState(display.STATISTICS_STATE);
        }});
    }
    protected void submitAnswer(int answer){
        submitTimer.cancel();
        lastAnswer = answer;
        display.setState(display.LOADING_STATE);
        display.setLoadingText("Submiting answer...");
        rpcService.submitAnswer(answer, submitAnswerCallback);
    }
    final AsyncCallback<QuestionHolder> getCurrentQuestionCallback = new AsyncCallback<QuestionHolder>(){

        public void onFailure(Throwable caught) {
            if (caught instanceof EndOfQuizException){
                display.setState(display.STATISTICS_END_STATE);
                questionTimer.cancel();
            }
            else {
                display.displayError("GetCurrentQuestion Failed", caught.getMessage(), basicCloseHandler);
            }
        }

        public void onSuccess(QuestionHolder result) {
            if (result == null){
                return;
            }

            display.getQuestionText().setText(result.getText());
            display.getAnswerAButton().setTitle(result.getAnswers().get(0).getText());
            answerAid = result.getAnswers().get(0).getId();
            display.getAnswerBButton().setTitle(result.getAnswers().get(1).getText());
            answerBid = result.getAnswers().get(1).getId();
            display.getAnswerCButton().setTitle(result.getAnswers().get(2).getText());
            answerCid = result.getAnswers().get(2).getId();
            display.getAnswerDButton().setTitle(result.getAnswers().get(3).getText());
            answerDid = result.getAnswers().get(3).getId();
            questionTimer.cancel();
            display.setState(display.QUESTION_STATE);
            submitTimer.schedule(result.getTimeleft()*1000);

        }

    };
    final AsyncCallback<List<StatisticsHolder>> getQuestionStatisticsCallback = new AsyncCallback<List<StatisticsHolder>>(){

        public void onFailure(Throwable caught) {
            display.displayError("GetQuestionStatistics Failed", caught.getMessage(), basicCloseHandler);
        }

        public void onSuccess(List<StatisticsHolder> result) {
            if (result == null){
                return;
            }
            statisticsTimer.cancel();
            for (StatisticsHolder sh : result){
                if ("correctAnswer".equals(sh.getKey())){
                    display.setState(display.STATISTICS_STATE);
                    if (lastAnswer == Integer.valueOf(sh.getValue())){
                        display.setStatistics("You answered correct", display.STATISTICS_GOOD);
                    }
                    else if (lastAnswer == -1){
                        display.setStatistics("You did not answer", display.STATISTICS_BAD);
                    }
                    else {
                        display.setStatistics("Your answer was wrong", display.STATISTICS_BAD);
                    }
                    break;
                }
            }
            questionTimer.scheduleRepeating(REFRESH_RATE);


        }
        
    };
    final CloseHandler basicCloseHandler = new CloseHandler() {
        public void onClose(CloseEvent event) {
                    //display.getLogInButton().setEnabled(true);
                }
    };

    final AsyncCallback<Boolean> submitAnswerCallback = new AsyncCallback<Boolean>(){

        public void onFailure(Throwable caught) {
            display.displayError("SubmitAnswer Failed", caught.getMessage(), basicCloseHandler);

            if (caught instanceof AnswerException){
                AnswerException ae = (AnswerException) caught;
                if (ae.getType() == AnswerException.INVALID_ID){
                    //Probbably missed a question, trying to get the current one.
                    display.setLoadingText("Trying to get the next question...");
                    questionTimer.scheduleRepeating(REFRESH_RATE);
                }
                lastAnswer = -1;
                display.setLoadingText("Waiting for question statistics...");
                statisticsTimer.scheduleRepeating(REFRESH_RATE);
            }
        }

        public void onSuccess(Boolean result) {
            display.setLoadingText("Waiting for question statistics...");
            statisticsTimer.scheduleRepeating(REFRESH_RATE);
        }

    };
        final AsyncCallback<List<StatisticsHolder>> getOverallStatisticsCallback = new AsyncCallback<List<StatisticsHolder>>(){

        public void onFailure(Throwable caught) {
            display.displayError("GetOverallStatistics Failed", caught.getMessage(), basicCloseHandler);
        }

        public void onSuccess(List<StatisticsHolder> result) {
            if (result == null){
                return;
            }
            statisticsTimer.cancel();
            Boolean found = false;
            for (StatisticsHolder sh : result){
                if ("correct".equals(sh.getKey())){
                    display.setState(display.STATISTICS_OVERALL_STATE);
                    // percentage of questions answered correct
                    Float value = Float.valueOf(sh.getValue());
                    NumberFormat nf = NumberFormat.getPercentFormat();
                    if (value>0.5){
                        display.setStatistics("You answered correct on " + nf.format((double)(value)) + " of the questions.", display.STATISTICS_GOOD);
                    }
                    else {
                        display.setStatistics("You answered correct on " + nf.format((double)(value)) + " of the questions.", display.STATISTICS_BAD);
                    }
                    found = true;
                    break;
                }
            }
            if (!found){
                display.displayError("Overall statistics error", "Could not find any usefull data in the statistics", basicCloseHandler);
            }

        }

    };

    /**
     * Method called when the user answered too late
     */
    protected void answerWasLate(){
            display.getQuestionText().setText("Too late...");
            lastAnswer = -1;
            submitTimer.cancel();
            display.setLoadingText("You did not answer in time. Waiting for question statistics...");
            display.setState(display.LOADING_STATE);
            statisticsTimer.scheduleRepeating(REFRESH_RATE);
    }
}
