/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.ntnu.idi.lqwebclient.server.service;

import com.google.gwt.user.server.rpc.RemoteServiceServlet;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.xml.namespace.QName;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.Holder;
import org.ntnu.idi.lqwebclient.server.ConfigurationManager;
import org.ntnu.idi.lqwebclient.client.exception.AnswerException;
import org.ntnu.idi.lqwebclient.client.exception.EndOfQuizException;
import org.ntnu.idi.lqwebclient.client.exception.GenericLQException;
import org.ntnu.idi.lqwebclient.client.gamemode.GameManager;
import org.ntnu.idi.lqwebclient.client.holder.AnswerHolder;
import org.ntnu.idi.lqwebclient.client.holder.QuestionHolder;
import org.ntnu.idi.lqwebclient.client.holder.StatisticsHolder;
import org.ntnu.idi.lqwebclient.client.holder.UserHolder;

import org.ntnu.idi.lqwebclient.client.service.LQservice;
import org.ntnu.idi.lqwebclient.shared.Answer;
import org.ntnu.idi.lqwebclient.shared.GameModeInfo;
import org.ntnu.idi.lqwebclient.shared.LectureQuizService;
import org.ntnu.idi.lqwebclient.shared.LectureQuizServiceService;
import org.ntnu.idi.lqwebclient.shared.QuestionInfo;
import org.ntnu.idi.lqwebclient.shared.StatisticsEntry;
import org.ntnu.idi.lqwebclient.server.User;

/**
 *
 * @author erlingbo
 */
public class LQserviceImpl extends RemoteServiceServlet implements LQservice {
    private LectureQuizServiceService service;

    private HttpSession getSession(){
        HttpServletRequest request = this.getThreadLocalRequest();
        return request.getSession();
    }

    private LectureQuizServiceService getService(){
        if (service != null ){
            return service;
        }
        URL wsdl = null;
        try
        {
            wsdl = new URL(ConfigurationManager.getInstance().getLQServerURL());
        } catch(MalformedURLException ex) {
            // TODO: Handle bad url
        }

        service = new LectureQuizServiceService(wsdl,new QName("http://service.server.lecturequiz/", "LectureQuizServiceService"));
        return service;
    }
    private void removeUser(){
        getSession().removeAttribute("User");
    }

    /**
     * Authenticates a user with the LectureQuiz server
     * @param username
     * @return Logged in username
     * @throws GenericLQException
     */
    public String authenticate(String username) throws GenericLQException {
        
        if (getSession().getAttribute("User") != null){
            //TODO: should do something with reauthing against the webservice
            return ((User)getSession().getAttribute("User")).getUsername();
        }
        if ("".equals(username) || username == null){
            throw new GenericLQException("Please fill in an username");
        }
        LectureQuizService port = getService().getLectureQuizServicePort();
        ((BindingProvider)port).getRequestContext().put(BindingProvider.SESSION_MAINTAIN_PROPERTY,true);
        int authRes = port.authenticate(username, null, false);

        if (authRes == 2){
            throw new GenericLQException("Username already in use. Please choose another one");
        }
        if (authRes == 0){
            //Success
            User user = new User(username,port);
            getSession().setAttribute("User", user);
            getSession().setMaxInactiveInterval(ConfigurationManager.getInstance().getSessionTimeoutInterval());
            return user.getUsername();
        }

        throw new UnknownError("Something unexpected happend in authenticate");
    }

    /**
     * Checks if the game mode for the given quiz is compatible with this client
     * @param quizcode
     * @return The id of the gamemode used by the given quiz
     */
    public String isGamemodeCompatible(String quizcode) throws GenericLQException{
        LectureQuizService port = (LectureQuizService) ((User)getSession().getAttribute("User")).getPort();
        if (port==null){
            removeUser();
            throw new GenericLQException("Unable to get port. Please try to login again.");
        }
        Holder<GameModeInfo> info = new Holder<GameModeInfo>();
        int retval = port.getGameModeInfo(quizcode, info);
        if (retval == 1){
            throw new GenericLQException("No such quiz game: " + quizcode);
        }
        GameModeInfo gmi = info.value;
        if (GameManager.getInstance().isGameModeSupported(gmi.getId())) { return gmi.getId(); }
        //else
        throw new GenericLQException("The game mode used in this quiz game is not supported by this client version");
    }



    public boolean joinQuiz(String quizcode) throws GenericLQException{
        LectureQuizService port = (LectureQuizService) ((User)getSession().getAttribute("User")).getPort();
        if (port == null){
            removeUser();
            throw new GenericLQException("Session failed: No port found. Try restarting, or get someone to fix this bug");
        }
        int retval = port.joinQuiz(quizcode,null);
        System.out.println("joinquizRetVal = " + retval);
        if ((retval==0) || (retval==5)){
            return true;
        }
        else if (retval == 4){
            throw new GenericLQException("You are not allowed to join this quiz");
        }
        else {
            throw new GenericLQException("An error occured while joining the quiz. Errorcode: " + retval);
        }

    }

    public QuestionHolder getCurrentQuestion() throws GenericLQException, EndOfQuizException{
        LectureQuizService port = (LectureQuizService) ((User)getSession().getAttribute("User")).getPort();
        if (port == null){
            removeUser();
            throw new GenericLQException("Session failed: No port found. Try restarting, or get someone to fix this bug");
        }
        Holder<QuestionInfo> info = new Holder<QuestionInfo>();
        int retval = port.getCurrentQuestion(info);
        if (retval == 4) {
            return null;
        }
        if (retval == 5){
            throw new EndOfQuizException();
        }
        if (retval != 0){
            throw new GenericLQException("An error occured while getting current question. Errorcode: " + retval);
        }
        QuestionHolder qh = new QuestionHolder();
        QuestionInfo qi = info.value;
        qh.setId(qi.getId());
        qh.setText(qi.getText());
        qh.setTimeleft(qi.getTimeleft());
        qh.setTimeout(qi.getTimeout());
        List AHList = qh.getAnswers();
        List answers = qi.getAnswers();
        for (Object ansObj : answers){
            Answer ans = (Answer) ansObj;
            AnswerHolder ah = new AnswerHolder(ans.getId(), ans.getText());
            AHList.add(ah);
        }
        return qh;
    }

    public List<StatisticsHolder> getQuestionStatistics() throws GenericLQException{
        LectureQuizService port = (LectureQuizService) ((User)getSession().getAttribute("User")).getPort();
        if (port == null){
            removeUser();
            throw new GenericLQException("Session failed: No port found. Try restarting, or get someone to fix this bug");
        }
        Holder<List<StatisticsEntry>> info = new Holder<List<StatisticsEntry>>();
        int retval = port.getQuestionStatistics(info);
        if (retval == 4) {
            return null;
        }
        if (retval != 0){
            throw new GenericLQException("An error occured while getting statistics. Errorcode: " + retval);
        }

        List<StatisticsHolder> returnList = new ArrayList<StatisticsHolder>();
        for (StatisticsEntry se : info.value){
            returnList.add(new StatisticsHolder(se.getKey(), se.getValue()));
        }
        return returnList;
    }

    public boolean submitAnswer(int answerId) throws GenericLQException, AnswerException{
        LectureQuizService port = (LectureQuizService) ((User)getSession().getAttribute("User")).getPort();
        if (port == null){
            removeUser();
            throw new GenericLQException("Session failed: No port found. Try restarting, or get someone to fix this bug");
        }
        int retval = port.submitAnswer(answerId);

        //The return values (4, 5 & 6) that says that the answer was not submitted, but the user is still in the game.
        if (retval == 4){
            AnswerException ae = new AnswerException("The answer that was provided does not belong to the current question. Maybe you missed one question?");
            ae.setType(AnswerException.INVALID_ID);
            throw ae;

        }
        if (retval == 5 ){
            AnswerException ae = new AnswerException("You have already answered on this question");
            ae.setType(AnswerException.ALREADY_ANSWERED);
            throw ae;
        }
        if (retval == 6){
            AnswerException ae = new AnswerException("Your answer was not accepted. You probbably answered to late.");
            ae.setType(AnswerException.NOT_ACCEPTED);
            throw ae;
        }


        if (retval != 0){
            throw new GenericLQException("An error occured while submiting an answer. Errorcode: " + retval);
        }

        return true;


    }
    public List<StatisticsHolder> getOverallStatistics() throws GenericLQException{
        LectureQuizService port = (LectureQuizService) ((User)getSession().getAttribute("User")).getPort();
        if (port == null){
            removeUser();
            throw new GenericLQException("Session failed: No port found. Try restarting, or get someone to fix this bug");
        }
        Holder<List<StatisticsEntry>> info = new Holder<List<StatisticsEntry>>();
        System.out.println("The OS Holder: " + info);
        int retval = port.getOverallStatistics(info);
        if (retval == 4) {
            throw new GenericLQException("Could not get the statistics at this time. Please try again");
        }
        if (retval != 0){
            throw new GenericLQException("An error occured while getting overall statistics. Errorcode: " + retval);
        }

        List<StatisticsHolder> returnList = new ArrayList<StatisticsHolder>();
        System.out.println("The Overall ServerList (" + info.value.size() + "): " + info.value);
        for (StatisticsEntry se : info.value){
            returnList.add(new StatisticsHolder(se.getKey(), se.getValue()));
        }
        System.out.println("The Overall List (" + returnList.size() + "): " + returnList);
        System.out.println("The Overall Entry: " + returnList.get(0).getKey() + " -- " + returnList.get(0).getValue());

        return returnList;
    }
    
    public UserHolder getUser() throws GenericLQException {
        User user = (User)getSession().getAttribute("User");
        if (user == null)
            return null;
        return user.asUserHolder();
    }

}
