/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package lecturequiz.server.data;

import java.util.ArrayList;
import lecturequiz.server.service.QuestionInfo;

/**
 *
 * @author erlingbo
 */
public class Question {
    private int id;
    private String text;
    private int timeout;
    private ArrayList<Answer> answers;
    
    private Answer correctAnswer;

    public Question(int id, String text, int timeout, 
            ArrayList<Answer> answers, Answer correctAnswer) {
        this.id = id;
        this.text = text;
        this.timeout = timeout;
        this.answers = (ArrayList<Answer>)answers.clone();
        this.correctAnswer = correctAnswer;
    }
    
    public boolean hasAnswer(int answerId){
        for (Answer answer : answers){
            if (answer.id == answerId){
                return true;
            }
        }
        return false;
    }

    public int getTimeout(){
        return timeout;
    }
    public int getID(){
        return id;
    }

    /**
     * Build a QuestionInfo object and return it.
     * @return An QuestionInfo object for this question.
     */
    public QuestionInfo getQuestionInfo() {
        QuestionInfo qi = new QuestionInfo();
        qi.id = id;
        qi.text = text;
        qi.timeout = timeout;
        qi.timeleft = timeout;
        qi.answers = new ArrayList<Answer>();

        for(int i = 0; i < answers.size(); i++)
            qi.answers.add(answers.get(i));

        return qi;
    }

    /**
     * Returns the correct answer to this question
     * @return the ID of the correct answer
     */
    public int getCorrectAnswer() {
        return correctAnswer.id;
    }

    /**
     * Returns the answers for this question
     * @return a list of all available answers
     */
    public ArrayList<Answer> getAnswers() {
        return answers;
    }

}
