/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package lecturequiz.server.main;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.InvalidPropertiesFormatException;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author avenger
 */
public class ConfigurationManager {
    private static ConfigurationManager _instance = null;

    // The preferences object used to store configurations on the computer
    private Properties props;

    // Keys used in the configration file
    private static final String DATABASE_HOST = "DatabaseHost";
    private static final String DATABASE_USER = "DatabaseUser";
    private static final String DATABASE_PASSWORD = "DatabasePassword";
    private static final String DATABASE_DB = "DatabaseDB";
    private static final String SESSION_TIMEOUT_INTERVAL = "SessionTimeoutInterval";

    private ConfigurationManager() {

        props = new Properties();
        
        try {
            InputStream is = this.getClass().getResourceAsStream("/configuration.xml");

            if(is != null) {
                props.loadFromXML(is);
            }
        } catch (FileNotFoundException ex) {
            Logger.getLogger(ConfigurationManager.class.getName()).log(Level.SEVERE, null, ex);
        } catch (InvalidPropertiesFormatException ex) {
            Logger.getLogger(ConfigurationManager.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(ConfigurationManager.class.getName()).log(Level.SEVERE, null, ex);
        }

        // Load all variables
        databaseHost = props.getProperty(DATABASE_HOST, "localhost");
        databaseUser = props.getProperty(DATABASE_USER, "lecturequiz");
        databasePassword = props.getProperty(DATABASE_PASSWORD, "lecturequiz");
        databaseDB = props.getProperty(DATABASE_DB, "lecturequiz");
        sessionTimeoutInterval = Integer.parseInt(props.getProperty(SESSION_TIMEOUT_INTERVAL, "1800"));
    }

    /**
     * Get an instance of the configuration settings
     * @return The singleton instance of this class.
     */
    public static synchronized ConfigurationManager getInstance() {
        if(_instance == null)
            _instance = new ConfigurationManager();
        return _instance;
    }

    private String databaseHost;
    public String getDatabaseHost() {
        return databaseHost;
    }

    public void setDatabaseHost(String host) {
        databaseHost = host;
    }

    private String databaseUser;
    public String getDatabaseUser() {
        return databaseUser;
    }

    public void setDatabaseUser(String user) {
        databaseUser = user;
    }

    private String databasePassword;
    public String getDatabasePassword() {
        return databasePassword;
    }

    public void setDatabasePassword(String password) {
        databasePassword = password;
    }

    private String databaseDB;
    public String getDatabaseDB() {
        return databaseDB;
    }

    public void setDatabaseDB(String db) {
        databaseDB = db;
    }

    private int sessionTimeoutInterval;
    public int getSessionTimeoutInterval() {
        return sessionTimeoutInterval;
    }

    public void setSessionTimeoutInterval(int interval) {
        sessionTimeoutInterval = interval;
    }
}

