/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package lecturequiz.server.main;

import java.util.ArrayList;
import java.util.List;
import lecturequiz.server.data.Quiz;
import lecturequiz.server.data.User;
import lecturequiz.server.service.FullQuestionInfo;
import lecturequiz.server.service.FullQuizInfo;
import lecturequiz.server.service.QuizInfo;

/**
 *
 * @author avenger
 */
public interface DBInterface {
    /**
     * Returns a list of all available quizzes
     * @return A list of QuizInfo objects containing information about a quiz or null if an error occurs.
     */
    public ArrayList<QuizInfo> getQuizList();

    /**
     * Tries to authenticate a user against the database.
     * @param username Username sent to the database.
     * @param password Password sent to the database.
     * @return User object with the authenticated user. null if authentication fails
     */
    public User authenticateUser(String username, String password);

    /**
     * Returns a user from the database with the supplied ID.
     * @param id ID of the user to retrieve
     * @return A User-object with teh requested user, or null if the user was not found
     */
    public User getUser(int id);

    /**
     * Returns the quiz with the given ID from the database
     * @param id ID of the quiz
     * @return Quiz with the given ID or null if the quiz was not found
     */
    public Quiz getQuiz(int id);

    /**
     * Returns a list of all available questions. The list will only contain the correct answer of a question
     * @return A list of FullQuestionInfo objects containing the information for all questions
     */
    public List<FullQuestionInfo> getQuestionList();

    /**
     * Returns all information on the question with the supplied id
     * @param id ID of the question to retrieve
     * @return A FullQuestionInfo object with all information, or null on error
     */
    public FullQuestionInfo getQuestion(int id);

    /**
     * Saves a quiz to the database
     * @param fqi All information of the quiz to be saved. If the ID of a quiz is 0
     * the quiz will be created. If any questions have the ID 0 they will be created and
     * answers of this question should be give ID's of 0-n and correctAnswer should point
     * to one of these IDs.
     * @param user The user trying to save the quiz. If this is not done by a logged in user
     * this can be null.
     * @return <b>Error codes:</b><br/>
     * 0 - Success<br/>
     * 1 - No such quiz<br/>
     * 2 - Invalid data<br/>
     * 3 - Access denied<br/>
     * 4 - Other error<br/>
     */
    public int saveQuiz(FullQuizInfo fqi,User user);
}
