/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package lecturequiz.server.main;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;
import lecturequiz.server.data.Answer;
import lecturequiz.server.data.Question;
import lecturequiz.server.data.Quiz;
import lecturequiz.server.data.User;
import lecturequiz.server.service.FullQuestionInfo;
import lecturequiz.server.service.FullQuizInfo;
import lecturequiz.server.service.QuizInfo;

/**
 * Implements a DB backend to MySQL
 * @author Knut A. Tidemann
 */
public class MySQLDB implements DBInterface {
    private Connection conn = null;
    private String dbhost;
    private String database;
    private String dbuser;
    private String dbpassword;

    /**
     * Private singleton constructor
     */
    public MySQLDB() {

        //TODO: Lage settingsfil
        dbhost = ConfigurationManager.getInstance().getDatabaseHost();
        database = ConfigurationManager.getInstance().getDatabaseDB();
        dbuser = ConfigurationManager.getInstance().getDatabaseUser();
        dbpassword = ConfigurationManager.getInstance().getDatabasePassword();

        // Register the MySQL driver
        try {
            Class.forName("com.mysql.jdbc.Driver").newInstance();
        } catch (Exception ex) {
            System.out.println("Could not add mysql driver: " + ex.getMessage());
        }

        try {
            conn = DriverManager.getConnection("jdbc:mysql://" + dbhost + "/" + database +
                    "?user=" + dbuser + "&password=" + dbpassword);
        } catch(SQLException ex) {
            System.out.println("Could not connect to database at " + dbhost + ":");
            System.out.println("SQLException: " + ex.getMessage());
            System.out.println("SQLState: " + ex.getSQLState());
            System.out.println("VendorError: " + ex.getErrorCode());

            conn = null;
        }
    }

    /**
     * Returns a list of all available quizzes
     * @return A list of QuizInfo objects containing information about a quiz or null if an error occurs.
     */
    public ArrayList<QuizInfo> getQuizList() {
        ArrayList<QuizInfo> list = new ArrayList<QuizInfo>();

        if(conn == null)
            return null;

        try
        {
            ResultSet rs = executeQuery("SELECT q.id,q.name,q.description,u.username FROM quiz q, user u WHERE q.owner = u.id");

            if(rs == null)
                return null;

            while(rs.next()) {
                QuizInfo qi = new QuizInfo();
                qi.id = rs.getInt("id");
                qi.name = rs.getString("name");
                qi.description = rs.getString("description");
                qi.owner = rs.getString("username");

                ResultSet qrs = executeQuery("SELECT COUNT(question) FROM ref_quiz_question WHERE quiz = '" + qi.id + "'");
                if(qrs != null && qrs.next()) {
                    qi.questionCount = qrs.getInt(1);
                    qrs.close();
                    qrs = null;
                }

                list.add(qi);
            }
        } catch(SQLException ex) {
            System.out.println("Could not build quiz list");
            System.out.println("SQLException: " + ex.getMessage());
            System.out.println("SQLState: " + ex.getSQLState());
            System.out.println("VendorError: " + ex.getErrorCode());

            return null;
        }

        return list;
    }

    /**
     * Tries to authenticate a user against the database.
     * @param username Username sent to the database.
     * @param password Password sent to the database.
     * @return User object with the authenticated user. null if authentication fails
     */
    public User authenticateUser(String username, String password) {
        PreparedStatement ps;
        ResultSet rs;
        User user;

        if(conn == null)
            return null;

        try
        {
            ps = conn.prepareStatement("SELECT id,username,role FROM user WHERE username = ? and password = SHA1(?)");
            ps.setString(1, username);
            ps.setString(2, password);

            rs = ps.executeQuery();

            if(!rs.next())
                return null;

            user = new User(rs.getInt("id"),rs.getString("username"),rs.getInt("role"));
        } catch(SQLException ex) {
            System.out.println("Could not authenticate user");
            System.out.println("SQLException: " + ex.getMessage());
            System.out.println("SQLState: " + ex.getSQLState());
            System.out.println("VendorError: " + ex.getErrorCode());

            return null;
        }

        return user;
    }

    /**
     * Returns a user from the database with the supplied ID.
     * @param id ID of the user to retrieve
     * @return A User-object with teh requested user, or null if the user was not found
     */
    public User getUser(int id) {

        User u;

        if(conn == null)
            return null;

        try
        {
            ResultSet rs = executeQuery("SELECT username,role FROM user WHERE id = '" + id + "'");

            if(rs == null || !rs.next())
                return null;

            u = new User(id,rs.getString("username"),rs.getInt("role"));

        } catch(SQLException ex) {
            System.out.println("Could not get user with id " + id + " from the database");
            System.out.println("SQLException: " + ex.getMessage());
            System.out.println("SQLState: " + ex.getSQLState());
            System.out.println("VendorError: " + ex.getErrorCode());

            return null;
        }

        return u;
    }

    /**
     * Returns the quiz with the given ID from the database
     * @param id ID of the quiz
     * @return Quiz with the given ID
     */
    public Quiz getQuiz(int id) {
        Quiz q;

        if(conn == null)
            return null;

        try
        {
            ResultSet rs = executeQuery("SELECT name,description,owner FROM quiz WHERE id = '" + id + "'");

            if(rs == null || !rs.next())
               return null;

            // Get questions
            ArrayList<Question> questions = new ArrayList<Question>();

            ResultSet rsq = executeQuery("SELECT id,`text`,timeout, correctAnswer " +
                   "FROM question q,ref_quiz_question r WHERE r.quiz = '" + id + "' and r.question = q.id");

            if(rsq != null)
            {
                while(rsq.next()) {
                    // Get the answers for this question
                    ResultSet rsa = executeQuery("SELECT id,`text` FROM answer WHERE question = '" + rsq.getInt("id") + "'");

                    // Skip this question if we could not get answers
                    if(rsa == null) {
                        System.out.println("Could not get answers for question with id " + rsq.getInt("id"));
                        continue;
                    }

                    ArrayList<Answer> answers = new ArrayList<Answer>();
                    Answer correctAnswer = null;

                    while(rsa.next()) {
                        Answer a = new Answer();
                        a.id = rsa.getInt("id");
                        a.text = rsa.getString("text");
                        answers.add(a);
                        if(a.id == rsq.getInt("correctAnswer"))
                            correctAnswer = a;
                    }

                    rsa.close();
                    rsa = null;

                    questions.add(new Question(rsq.getInt("id"),rsq.getString("text"),
                            rsq.getInt("timeout"),answers, correctAnswer));
                }
            }

            rsq.close();
            rsq = null;

            // Get the owner
            User u = getUser(rs.getInt("owner"));

            // Build the quiz object
            q = new Quiz(id, rs.getString("name"), rs.getString("description"), questions, u);

        } catch(SQLException ex) {
            logSQLError("Could not build quiz object with ID " + id + " from the database",ex);

            return null;
        }



        return q;
    }

    /**
     * Returns a list of all available questions. The list will only contain the correct answer of a question
     * @return A list of FullQuestionInfo objects containing the information for all questions
     */
    public List<FullQuestionInfo> getQuestionList() {
        List<FullQuestionInfo> questions = new ArrayList<FullQuestionInfo>();

        if(conn == null)
            return null;

        try {
            ResultSet res = executeQuery("SELECT q.`id`,q.`text`,q.`timeout`,q.`correctAnswer`,a.`text` FROM " +
                    "`question` q, `answer` a WHERE q.`correctAnswer` = a.`id`");

            while(res.next()) {
                FullQuestionInfo fqi = new FullQuestionInfo();
                fqi.id = res.getInt(1);
                fqi.text = res.getString(2);
                fqi.timeleft = fqi.timeout = res.getInt(3);
                fqi.correctAnswer = res.getInt(4);
                fqi.answers = new ArrayList<Answer>();
                Answer a = new Answer();
                a.id = fqi.correctAnswer;
                a.text = res.getString(5);
                fqi.answers.add(a);

                questions.add(fqi);
            }
        } catch(SQLException ex) {
            logSQLError("Could not get question list", ex);
            return null;
        }

        return questions;
    }

    /**
     * Returns all information on the question with the supplied id
     * @param id ID of the question to retrieve
     * @return A FullQuestionInfo object with all information, or null on error
     */
    public FullQuestionInfo getQuestion(int id) {
        FullQuestionInfo fqi = new FullQuestionInfo();
        ResultSet res;

        if(conn == null)
            return null;

        try {
            res = executeQuery("SELECT `text`,`timeout`,`correctAnswer` FROM `question` WHERE `id` = '" + id + "'");
            if(!res.next())
                return null;

            fqi.id = id;
            fqi.text = res.getString(1);
            fqi.timeleft = fqi.timeout = res.getInt(2);
            fqi.correctAnswer = res.getInt(3);
            fqi.answers = new ArrayList<Answer>();

            res = executeQuery("SELECT `id`,`text` FROM `answer` WHERE `question` = '" + id + "'");
            while(res.next()) {
                Answer a = new Answer();
                a.id = res.getInt(1);
                a.text = res.getString(2);
                fqi.answers.add(a);
            }

        } catch(SQLException ex) {
            logSQLError("Could not get question with id: " + id, ex);
            return null;
        }

        return fqi;
    }

    /**
     * Saves a quiz to the database
     * @param fqi All information of the quiz to be saved. If the ID of a quiz is 0
     * the quiz will be created. If any questions have the ID 0 they will be created and
     * answers of this question should be give ID's of 0-n and correctAnswer should point
     * to one of these IDs.
     * @param user The user trying to save the quiz. If this is not done by a logged in user
     * this can be null.
     * @return <b>Error codes:</b><br/>
     * 0 - Success<br/>
     * 1 - No such quiz<br/>
     * 2 - Invalid data<br/>
     * 3 - Access denied<br/>
     * 4 - Other error<br/>
     */
    public int saveQuiz(FullQuizInfo fqi, User user) {
        if(conn == null)
            return 4;

        // Verify data
        ResultSet res = null;

        // Make sure the quiz exists
        if(fqi.id != 0) {
            try {
                res = executeQuery("SELECT owner FROM quiz WHERE id = '" + fqi.id + "'");
            } catch(SQLException ex) {
                logSQLError("saveQuiz: could not lookup quiz id", ex);
                return 4;
            }

            try {
                if(!res.next())
                    return 1;

                if(user != null && user.getRole() != User.ADMIN && user.getId() != res.getInt(1))
                    return 3;
                
            } catch(SQLException ex) {
                logSQLError("saveQuiz: Could not lookup quiz id", ex);
                return 4;
            }
        }

        // Make sure all supplied questions and answers exist
        // Loop though all questions which supposedly exists
        ArrayList<Integer> questions = new ArrayList<Integer>();
        ArrayList<Integer> answers = new ArrayList<Integer>();

        for(int i = 0; i < fqi.questions.size(); i++) {
            if(fqi.questions.get(i).id != 0)
                questions.add(fqi.questions.get(i).id);

            // Check the answers of this quiz
            boolean hasCorrectAnswer = false;
            for(int j = 0; j < fqi.questions.get(i).answers.size(); j++) {
                int aid = fqi.questions.get(i).answers.get(j).id;
                if(aid == fqi.questions.get(i).correctAnswer)
                    hasCorrectAnswer = true;

                if(fqi.questions.get(i).id != 0) // Don't check answers if it's a new question
                    answers.add(aid);
            }

            if(!hasCorrectAnswer)
                return 2;
        }

        // Make a DB lookup to make sure all questions exist
        String sql;
        
        if(questions.size() > 0) {
            sql = "SELECT COUNT(id) FROM question WHERE id IN (";
            for(int i = 0; i < questions.size(); i++) {
                sql += questions.get(i) + ",";
            }
            sql = sql.substring(0, sql.length() - 1) + ")";

            try {
                res = executeQuery(sql);

                if(!res.next())
                    return 4;

                if(res.getInt(1) != questions.size())
                    return 2;

            } catch(SQLException ex) {
                logSQLError("saveQuiz: Could not validate questions.", ex);
                return 4;
            }
        }

        if(answers.size() > 0) {
            // Now answers
            sql = "SELECT COUNT(id) FROM answer WHERE id IN (";
            for(int i = 0; i < answers.size(); i++) {
                sql += answers.get(i) + ",";
            }
            sql = sql.substring(0, sql.length() - 1) + ")";

            try {
                res = executeQuery(sql);

                if(!res.next())
                    return 4;

                if(res.getInt(1) != answers.size())
                    return 2;

            } catch(SQLException ex) {
                logSQLError("saveQuiz: Could not validate answers.", ex);
                return 4;
            }
        }

        // Now to the saving of the data
        // Save all questions
        for(int i = 0; i < fqi.questions.size(); i++) {
            if(fqi.questions.get(i).id == 0) {
                fqi.questions.get(i).id = newQuestion(fqi.questions.get(i));
                if(fqi.questions.get(i).id == 0)
                    return 4;
            } else {
                if(!saveQuestion(fqi.questions.get(i)))
                    return 4;
            }
        }

        // Create or update the quiz
        if(fqi.id == 0) {
            try {
                PreparedStatement s = conn.prepareStatement("INSERT INTO `quiz` (`name`,`description`,`owner`) VALUES(?,?,?)",
                        Statement.RETURN_GENERATED_KEYS);

                s.setString(1, fqi.name);
                s.setString(2, fqi.description);
                s.setInt(3, user.getId());
                s.executeUpdate();

                res = s.getGeneratedKeys();
                if(!res.next())
                    return 4;

                fqi.id = res.getInt(1);
            } catch(SQLException ex) {
                logSQLError("Could not create quiz", ex);
                return 4;
            }
        } else {
            try
            {
                PreparedStatement s = conn.prepareStatement("UPDATE `quiz` SET `name` = ?, `description` = ? WHERE id = ?");
                s.setString(1, fqi.name);
                s.setString(2, fqi.description);
                s.setInt(3, fqi.id);
            } catch(SQLException ex) {
                logSQLError("Could not update quiz", ex);
                return 4;
            }
        }

        // Update the relationslist between a quiz and its questions.
        // First delete all old ones
        try {
            executeUpdate("DELETE FROM `ref_quiz_question` WHERE `quiz` = '" + fqi.id + "'");
        } catch(SQLException ex) {
            logSQLError("Could not delete quiz-question references", ex);
            return 4;
        }

        // Insert new ones
        for(int i = 0; i < fqi.questions.size(); i++) {
            try {
                executeUpdate("INSERT INTO `ref_quiz_question` (`quiz`,`question`) VALUES('" +
                        fqi.id + "','" + fqi.questions.get(i).id +"')");
            } catch(SQLException ex) {
                logSQLError("Could not create quiz-question references", ex);
                return 4;
            }
        }

        return 0;
    }

    /**
     * Creates a new question in the database
     * @param fqi The information about the questions
     * @return The ID of the new question, 0 on failure
     */
    private int newQuestion(FullQuestionInfo fqi) {

        // Add the question
        int qid;
        try {
            PreparedStatement s = conn.prepareStatement("INSERT INTO question (`text`,timeout,correctAnswer) VALUES(?,?,0)",
                    Statement.RETURN_GENERATED_KEYS);
            s.setString(1, fqi.text);
            s.setInt(2, fqi.timeout);
            s.executeUpdate();
            ResultSet res = s.getGeneratedKeys();

            if(!res.next())
                return 0;

            qid = res.getInt(1);
        } catch(SQLException ex) {
            logSQLError("Could not insert question to database", ex);
            return 0;
        }

        // Add all answers
        int aid = 0;
        for(int i = 0; i < fqi.answers.size(); i++) {
            try {
                PreparedStatement s = conn.prepareStatement("INSERT INTO answer (`text`,question) VALUES(?,?)",
                        Statement.RETURN_GENERATED_KEYS);
                s.setString(1, fqi.answers.get(i).text);
                s.setInt(2, qid);
                s.executeUpdate();

                if(fqi.answers.get(i).id == fqi.correctAnswer) {
                    ResultSet res = s.getGeneratedKeys();
                    if(!res.next())
                        return 0;

                    aid = res.getInt(1);
                }

            } catch(SQLException ex) {
                logSQLError("Could not insert answer into database", ex);
                return 0;
            }
        }

        // Update the question row so it points to the correct answer
        try {
            executeUpdate("UPDATE question SET correctAnswer = '" + aid + "' WHERE id = '" + qid + "'");
        } catch(SQLException ex) {
            logSQLError("Could not update correctAnswer on quiz", ex);
            return 0;
        }

        return qid;
    }

    /**
     * Saves the changes to a question in the database
     * @param fqi Updated question information
     * @return True on success, false on error
     */
    private boolean saveQuestion(FullQuestionInfo fqi) {

        // Update question
        try {
            PreparedStatement s = conn.prepareStatement("UPDATE question SET `text` = ?, timeout = ?, correctAnswer = ? WHERE id = ?");
            s.setString(1, fqi.text);
            s.setInt(2, fqi.timeout);
            s.setInt(3,fqi.correctAnswer);
            s.setInt(4, fqi.id);
            s.executeUpdate();
        } catch(SQLException ex) {
            logSQLError("Could not update question", ex);
            return false;
        }

        // Update answers
        for( int i = 0; i < fqi.answers.size(); i++) {
            try {
                PreparedStatement s = conn.prepareStatement("UPDATE answer SET `text` = ? WHERE id = ?");
                s.setString(1, fqi.answers.get(i).text);
                s.setInt(2, fqi.answers.get(i).id);
                s.executeUpdate();
            } catch(SQLException ex) {
                logSQLError("Could not update question", ex);
                return false;
            }
        }

        return true;
    }

    /**
     * Helper function to execute a query.
     * @param query Query to execute.
     * @return ResultSet with the result of the query.
     * @throws SQLException Throws on SQL error.
     */
    private ResultSet executeQuery(String query) throws SQLException {
        Statement s = null;
        ResultSet rs = null;

        s = conn.createStatement();
        rs = s.executeQuery(query);

        return rs;
    }

    /**
     * Helper function to execute a query with no result set
     * @param query Query to execute.
     * @throws SQLException Throws on SQL error.
     */
    private void executeUpdate(String query) throws SQLException {
        Statement s = null;

        s = conn.createStatement();
        s.executeUpdate(query);
    }

    private void logSQLError(String customMessage, SQLException ex) {
        if(!customMessage.isEmpty())
            System.out.println(customMessage);
        
        System.out.println("SQLException: " + ex.getMessage());
        System.out.println("SQLState: " + ex.getSQLState());
        System.out.println("VendorError: " + ex.getErrorCode());
    }
}
