/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package lecturequiz.server.main;

import java.util.ArrayList;
import lecturequiz.server.data.User;

/**
 *
 * @author avenger
 */
public class UserManager {

    /// Singleton instance
    private static UserManager _instance = null;
    /// The list of all users
    private ArrayList<User> users;

    /**
     * Constructor
     */
    private UserManager() {
        users = new ArrayList<User>();
    }

    /**
     * Returns the instance of the UserManager class, or creates a new on if it
     * doesn't exist.
     * @return The UserManager instance;
     */
    public static synchronized UserManager getInstance() {
        if(_instance == null) {
            _instance = new UserManager();
        }

        return _instance;
    }

    /**
     * Checks if the supplied username is already in use by another logged in user.
     * @param username The name to check availability for.
     * @return True if the name is available, false if not.
     */
    public boolean isUsernameAvailable(String username) {
        for(int i = 0; i < users.size(); i++) {
            if(users.get(i).getNickname().toLowerCase().equals(username.toLowerCase()))
                return false;
        }

        return true;
    }

    /**
     * Adds a user to the user-pool.
     * @param username The username of the user to be added.
     * @param role The role of the user to be added.
     * @return The User-object created.
     */
    public User addUser(String username, int role) {
        User u = new User(username,role);
        users.add(u);
        return u;
    }

    /**
     * Adds a user to the list of users
     * @param user the user to add
     */
    public void addUser(User user) {
        users.add(user);
    }

    /**
     * Removes a user from the list of users and any games they are in
     * @param user the user to remove
     */
    public void removeUser(User user) {

        if(user.getCurrentGame() != null) {
            if(user.getCurrentGame().getOwner() == user) {
                // Since the role is a higher one, this user is running a game
                GameManager.getInstance().endGame(user.getCurrentGame());
            }else {
                // Make sure the user is removed from any games it is participating in
                user.getCurrentGame().removeUser(user);
                user.setCurrentGame(null);
            }
        }
        
        users.remove(user);
    }

    /**
     * Tries to authenticate the user with the
     * @param username Username to authenticate
     * @param password Password
     * @return A User object or null if authentication failed
     */
    public User authenticateUser(String username, String password) {
        return DBManager.getInstance().authenticateUser(username, password);
    }
}
